@@ -3,7 +3,7 @@ from collections import deque
 from dataclasses import dataclass
 from functools import partial
 import logging
-from typing import Deque, List, Optional, Union, Tuple
+from typing import Optional, Union
 
 from .base_manager import BaseManager
 from .constants import (
@@ -25,6 +25,7 @@ from .protocol.messages import (
     AcceptChildren,
     BranchLevel,
     BranchRoot,
+    ChildDepth,
     DistributedAliveInterval,
     DistributedBranchLevel,
     DistributedBranchRoot,
@@ -62,13 +63,10 @@ logger = logging.getLogger(__name__)
 class DistributedPeer:
     """Represents a distributed peer and its values in the distributed network"""
     username: str
-    connection: Optional[PeerConnection] = None
-    """Distributed connection (type=D) associated with the peer. A `None` value
-    indicates the peer represents the current client
-    """
+    connection: PeerConnection
+    """Distributed connection (type=D) associated with the peer"""
     branch_level: Optional[int] = None
     branch_root: Optional[str] = None
-    child_depth: Optional[int] = None
 
 
 class DistributedNetwork(BaseManager):
@@ -86,10 +84,10 @@ class DistributedNetwork(BaseManager):
         """Distributed parent. This variable is `None` if we are looking for a
         parent
         """
-        self.children: List[DistributedPeer] = []
-        self.potential_parents: Deque[str] = deque(
+        self.children: list[DistributedPeer] = []
+        self.potential_parents: deque[str] = deque(
             maxlen=POTENTIAL_PARENTS_CACHE_SIZE)
-        self.distributed_peers: List[DistributedPeer] = []
+        self.distributed_peers: list[DistributedPeer] = []
 
         # State parameters sent by the server
         self.parent_min_speed: Optional[int] = None
@@ -104,7 +102,7 @@ class DistributedNetwork(BaseManager):
 
         self.register_listeners()
 
-        self._potential_parent_tasks: List[asyncio.Task] = []
+        self._potential_parent_tasks: list[asyncio.Task] = []
 
     def register_listeners(self):
         self._event_bus.register(
@@ -118,7 +116,7 @@ class DistributedNetwork(BaseManager):
         self._event_bus.register(
             SessionDestroyedEvent, self._on_session_destroyed)
 
-    def _get_advertised_branch_values(self) -> Tuple[str, int]:
+    def _get_advertised_branch_values(self) -> tuple[str, int]:
         """Returns the advertised branch values. These values are to be sent to
         the children and the server to let them know where we are in the
         distributed tree.
@@ -178,7 +176,7 @@ class DistributedNetwork(BaseManager):
 
         :param peer: Peer that should become our parent
         """
-        logger.info(f"set parent : {peer}")
+        logger.info("set parent : %s", peer)
         self.parent = peer
 
         await asyncio.gather(
@@ -189,7 +187,7 @@ class DistributedNetwork(BaseManager):
         # be disconnected
         distributed_connections = [
             dpeer.connection for dpeer in self.distributed_peers
-            if dpeer in [self.parent, ] + self.children and dpeer.connection is not None
+            if dpeer in [self.parent, ] + self.children
         ]
 
         disconnect_tasks = []
@@ -214,8 +212,7 @@ class DistributedNetwork(BaseManager):
             if not self.parent:
                 await self._set_parent(peer)
             else:
-                if peer.connection:
-                    await peer.connection.disconnect(reason=CloseReason.REQUESTED)
+                await peer.connection.disconnect(reason=CloseReason.REQUESTED)
 
     async def _disconnect_children(self):
         await asyncio.gather(
@@ -224,18 +221,14 @@ class DistributedNetwork(BaseManager):
         )
 
     async def _disconnect_child(self, peer: DistributedPeer):
-        if peer.connection:
-            await peer.connection.disconnect(CloseReason.REQUESTED)
+        await peer.connection.disconnect(CloseReason.REQUESTED)
 
     async def _disconnect_parent(self):
         if self.parent:
-            if self.parent.connection:
-                await self.parent.connection.disconnect(CloseReason.REQUESTED)
-            else:
-                await self._unset_parent()
+            await self.parent.connection.disconnect(CloseReason.REQUESTED)
 
     async def _unset_parent(self):
-        logger.debug(f"unset parent {self.parent!r}")
+        logger.info("unset parent : %s", self.parent)
 
         self.parent = None
 
@@ -258,12 +251,17 @@ class DistributedNetwork(BaseManager):
         server that we are looking for one
         """
         root, level = self._get_advertised_branch_values()
-        logger.info(f"notifying server of our parent : level={level} root={root}")
+        logger.info("notifying server of our parent : level=%d root=%s", level, root)
+
+        search_for_parent = False if self.parent else True
+
+        if not self._settings.debug.search_for_parent:
+            search_for_parent = False
 
         await self._network.send_server_messages(*[
             BranchLevel.Request(level),
             BranchRoot.Request(root),
-            ToggleParentSearch.Request(False if self.parent else True)
+            ToggleParentSearch.Request(search_for_parent)
         ])
 
     async def _notify_children_of_branch_values(self):
@@ -280,18 +278,16 @@ class DistributedNetwork(BaseManager):
             return
 
         if not self._accept_children:
-            logger.debug(f"not accepting children, rejecting peer as child: {peer}")
-            if peer.connection:  # Satisfy type checker
-                await peer.connection.disconnect(CloseReason.REQUESTED)
+            logger.debug("not accepting children, rejecting peer as child : %s", peer)
+            await peer.connection.disconnect(CloseReason.REQUESTED)
             return
 
         if len(self.children) >= self._max_children:
             logger.debug(
-                f"maximum amount of children reached ({len(self.children)} / {self._max_children}), "
-                f"rejecting peer as child: {peer}"
+                "maximum amount of children reached (%d / %d), rejecting peer as child : %s",
+                len(self.children), self._max_children, peer
             )
-            if peer.connection:  # Satisfy type checker
-                await peer.connection.disconnect(CloseReason.REQUESTED)
+            await peer.connection.disconnect(CloseReason.REQUESTED)
             return
 
         await self._add_child(peer)
@@ -300,18 +296,28 @@ class DistributedNetwork(BaseManager):
         if not peer.connection:
             return
 
-        logger.debug(f"adding distributed connection as child : {peer!r}")
         self.children.append(peer)
         # Let the child know where we are in the distributed tree
         root, level = self._get_advertised_branch_values()
 
         await peer.connection.send_message(DistributedBranchLevel.Request(level))
-        await peer.connection.send_message(DistributedBranchRoot.Request(root))
+        if level != 0:
+            await peer.connection.send_message(DistributedBranchRoot.Request(root))
+
+        logger.debug(
+            "added distributed connection as child (%d / %d children) : %s",
+            len(self.children), self._max_children, peer
+        )
 
     def _remove_child(self, peer: DistributedPeer):
-        logger.info(f"removing child : {peer!r}")
+        logger.info("removing child : %r", peer)
         self.children.remove(peer)
 
+        logger.debug(
+            "removed distributed connection as child (%d / %d children) : %s",
+            len(self.children), self._max_children, peer
+        )
+
     def _potential_parent_task_callback(self, username: str, task: asyncio.Task):
         """Callback for potential parent handling task. This callback simply
         logs the results and removes the task from the list
@@ -320,11 +326,11 @@ class DistributedNetwork(BaseManager):
             task.result()
 
         except asyncio.CancelledError:
-            logger.debug(f"request for potential parent cancelled (username={username})")
+            logger.debug("request for potential parent cancelled (username=%s)", username)
         except Exception as exc:
-            logger.warning(f"request for potential parent failed : {exc!r} (username={username})")
+            logger.warning("request for potential parent failed : %r (username=%s)", exc, username)
         else:
-            logger.info(f"request for potential parent successful (username={username})")
+            logger.info("request for potential parent successful (username=%s)", username)
         finally:
             self._potential_parent_tasks.remove(task)
 
@@ -393,16 +399,6 @@ class DistributedNetwork(BaseManager):
             if message.username == username:
                 return
 
-            if not self.parent:
-                # Set ourself as parent
-                parent = DistributedPeer(
-                    username,
-                    None,
-                    branch_root=username,
-                    branch_level=0
-                )
-                await self._set_parent(parent)
-
         await self.send_messages_to_children(message)
 
     @on_message(ResetDistributed.Response)
@@ -414,11 +410,11 @@ class DistributedNetwork(BaseManager):
     @on_message(DistributedBranchLevel.Request)
     async def _on_distributed_branch_level(
             self, message: DistributedBranchLevel.Request, connection: PeerConnection):
-        logger.info(f"branch level {message.level!r}: {connection!r}")
+        logger.info("branch level %r : %r", message.level, connection)
 
         peer = self.get_distributed_peer(connection)
         if not peer:
-            logger.warning(f"distributed peer object not found for connection: {connection} : {message}")
+            logger.warning("distributed peer object not found for connection: %s : %s", connection, message)
             return
 
         peer.branch_level = message.level
@@ -431,7 +427,7 @@ class DistributedNetwork(BaseManager):
         if peer != self.parent:
             await self._check_if_new_parent(peer)
         else:
-            logger.info(f"parent advertised new branch level : {message.level}")
+            logger.info("parent advertised new branch level : %d", message.level)
             await self._notify_children_of_branch_values()
 
     @on_message(DistributedBranchRoot.Request)
@@ -441,7 +437,7 @@ class DistributedNetwork(BaseManager):
 
         peer = self.get_distributed_peer(connection)
         if not peer:
-            logger.warning(f"distributed peer object not found for connection: {connection} : {message}")
+            logger.warning("distributed peer object not found for connection: %s : %s", connection, message)
             return
 
         # When we receive branch level 0 we automatically assume the root is the
@@ -454,29 +450,40 @@ class DistributedNetwork(BaseManager):
         if peer != self.parent:
             await self._check_if_new_parent(peer)
         else:
-            logger.info(f"parent advertised new branch root : {message.username}")
+            logger.info("parent advertised new branch root : %s", message.username)
             await self._notify_children_of_branch_values()
 
     @on_message(DistributedChildDepth.Request)
     async def _on_distributed_child_depth(
             self, message: DistributedChildDepth.Request, connection: PeerConnection):
+
         peer = self.get_distributed_peer(connection)
         if not peer:
-            logger.warning(f"distributed peer object not found for connection: {connection} : {message}")
+            logger.warning("distributed peer object not found for connection: %s : %s", connection, message)
             return
 
-        peer.child_depth = message.depth
+        # Not essential but might be needed for backward compatibility
+        if self.parent and self.parent.connection:
+            await self.parent.connection.send_message(
+                DistributedChildDepth.Request(message.depth + 1)
+            )
+        else:
+            await self._network.send_server_messages(
+                ChildDepth.Request(message.depth + 1)
+            )
 
     @on_message(DistributedSearchRequest.Request)
     async def _on_distributed_search_request(
             self, message: DistributedSearchRequest.Request, connection: PeerConnection):
+
         await self.send_messages_to_children(message)
 
     @on_message(DistributedServerSearchRequest.Request)
     async def _on_distributed_server_search_request(
             self, message: DistributedServerSearchRequest.Request, connection: PeerConnection):
+
         if message.distributed_code != DistributedSearchRequest.Request.MESSAGE_ID:
-            logger.warning(f"no handling for server search request with code {message.distributed_code}")
+            logger.warning("no handling for server search request with code : %d", message.distributed_code)
             return
 
         dmessage = DistributedSearchRequest.Request(
@@ -493,15 +500,13 @@ class DistributedNetwork(BaseManager):
             speed = message.user_stats.avg_speed
 
             if self.parent_min_speed is None:
-                logger.debug(
-                    f"using default parent_min_speed: {DEFAULT_PARENT_MIN_SPEED}")
+                logger.debug("using default parent_min_speed: %d", DEFAULT_PARENT_MIN_SPEED)
                 parent_min_speed = DEFAULT_PARENT_MIN_SPEED
             else:
                 parent_min_speed = self.parent_min_speed
 
             if self.parent_speed_ratio is None:
-                logger.debug(
-                    f"using default parent_speed_ratio: {DEFAULT_PARENT_SPEED_RATIO}")
+                logger.debug("using default parent_speed_ratio: %d", DEFAULT_PARENT_SPEED_RATIO)
                 parent_speed_ratio = DEFAULT_PARENT_SPEED_RATIO
             else:
                 parent_speed_ratio = self.parent_speed_ratio
@@ -515,16 +520,16 @@ class DistributedNetwork(BaseManager):
                 self._max_children = self._calculate_max_children(speed, parent_speed_ratio)
 
             logger.debug(
-                f"adjusting distributed children values: "
-                f"accept={self._accept_children}, max_children={self._max_children}"
+                "adjusting distributed child values: accept=%d, max_children=%d",
+                self._accept_children, self._max_children
             )
             await self._network.send_server_messages(
                 AcceptChildren.Request(self._accept_children)
             )
 
     def _has_parent_speed_values(self) -> bool:
-        """Returns `True` if `ParentMinSpeed` and `ParentSpeedRatio` has been
-        received from the server
+        """Returns ``True`` if :class:`.ParentMinSpeed` and
+        :class:`.ParentSpeedRatio` have been received from the server
         """
         return self.parent_min_speed is not None and self.parent_speed_ratio is not None
 
@@ -584,7 +589,7 @@ class DistributedNetwork(BaseManager):
                 # occur
                 if not peer:
                     logger.warning(
-                        f"connection was not registered with the distributed network : {connection!r}")
+                        "connection was not registered with the distributed network : %r", connection)
                     return
 
                 # Check if it was the parent or child that was disconnected
@@ -599,19 +604,13 @@ class DistributedNetwork(BaseManager):
 
         elif isinstance(connection, ServerConnection):
 
-            # When the client itself is branch root it means that the server is
-            # our "parent" and must be unset here
-            if self.parent and self.parent.connection is None:
-                await self._unset_parent()
-
             self._reset_server_values()
 
     async def send_messages_to_children(self, *messages: Union[MessageDataclass, bytes]):
         for child in self.children:
-            if child.connection:
-                child.connection.queue_messages(*messages)
+            child.connection.queue_messages(*messages)
 
-    async def stop(self) -> List[asyncio.Task]:
+    async def stop(self) -> list[asyncio.Task]:
         """Cancels all pending tasks
 
         :return: a list of tasks that have been cancelled so that they can be
@@ -619,7 +618,7 @@ class DistributedNetwork(BaseManager):
         """
         return self._cancel_potential_parent_tasks()
 
-    def _cancel_potential_parent_tasks(self) -> List[asyncio.Task]:
+    def _cancel_potential_parent_tasks(self) -> list[asyncio.Task]:
         cancelled_tasks = []
 
         for task in self._potential_parent_tasks:
