@@ -1,24 +1,15 @@
 from __future__ import annotations
 import asyncio
-from dataclasses import dataclass
+from collections.abc import Callable, Coroutine
+from dataclasses import dataclass, field
 import inspect
 import logging
-from typing import (
-    Any,
-    Callable,
-    Coroutine,
-    Dict,
-    List,
-    Optional,
-    Tuple,
-    Type,
-    TypeVar,
-    TYPE_CHECKING,
-    Union
-)
+from typing import Any, Optional, TypeVar, TYPE_CHECKING, Union
+from types import MethodType
+import weakref
 
 from .room.model import Room, RoomMessage
-from .user.model import ChatMessage, User
+from .user.model import BlockingFlag, ChatMessage, User
 from .protocol.primitives import (
     DirectoryData,
     MessageDataclass,
@@ -65,6 +56,7 @@ from .protocol.messages import (
     UserLeftRoom,
 )
 from .search.model import SearchRequest, SearchResult
+from .shares.model import SharedDirectory
 from .session import Session
 
 if TYPE_CHECKING:
@@ -91,8 +83,9 @@ EventListener = Union[
 
 # Internal functions
 
-def on_message(message_class: Type[MessageDataclass]):
-    """Decorator for methods listening to specific `MessageData` events"""
+def on_message(message_class: type[MessageDataclass]):
+    """Decorator for methods listening to specific :class:`.MessageData` events
+    """
     def register(event_func):
         event_func._registered_message = message_class
         return event_func
@@ -100,9 +93,9 @@ def on_message(message_class: Type[MessageDataclass]):
     return register
 
 
-def build_message_map(obj: object) -> Dict[Type[MessageDataclass], Callable]:
+def build_message_map(obj: object) -> dict[type[MessageDataclass], Callable]:
     methods = inspect.getmembers(obj, predicate=inspect.ismethod)
-    mapping: Dict[Type[MessageDataclass], Callable] = {}
+    mapping: dict[type[MessageDataclass], Callable] = {}
     for _, method in methods:
         registered_message = getattr(method, '_registered_message', None)
         if registered_message:
@@ -115,10 +108,23 @@ def build_message_map(obj: object) -> Dict[Type[MessageDataclass], Callable]:
 class EventBus:
 
     def __init__(self):
-        self._events: Dict[Type[Event], List[Tuple[int, EventListener]]] = {}
+        self._events: dict[
+            type[Event],
+            list[tuple[int, weakref.ReferenceType[EventListener]]]] = {}
 
-    def register(self, event_class: Type[E], listener: EventListener, priority: int = 100):
-        entry = (priority, listener)
+    def register(self, event_class: type[E], listener: EventListener, priority: int = 100):
+        """Registers an event listener to listen on an event class. The order in
+        which the listeners are called can be managed using the ``priority``
+        parameter
+        """
+        ref_factory: type[weakref.ReferenceType] = weakref.ref
+        if isinstance(listener, MethodType):
+            ref_factory = weakref.WeakMethod
+
+        entry = (
+            priority,
+            ref_factory(listener, lambda ref: self._remove_callback(event_class, ref))
+        )
         try:
             self._events[event_class].append(entry)
         except KeyError:
@@ -126,20 +132,81 @@ class EventBus:
 
         self._events[event_class].sort(key=lambda e: e[0])
 
+    def unregister(self, event_class: type[E], listener: EventListener):
+        """Unregisters the event listener from event bus"""
+        if event_class not in self._events:
+            return
+
+        self._events[event_class] = [
+            listener_def
+            for listener_def in self._events[event_class]
+            if listener != listener_def[1]()
+        ]
+
     async def emit(self, event: Event):
+        """Emit an event on the event bus. Listeners can be co-routines or
+        regular functions
+
+        :param event: the event instance to emit
+        """
+        for listener in self._get_listeners_for_event(event):
+            try:
+                if asyncio.iscoroutinefunction(listener):
+                    await listener(event)
+                else:
+                    listener(event)
+
+            except Exception:
+                logger.exception(
+                    "exception notifying listener %r of event %r",
+                    listener, event
+                )
+
+    def emit_sync(self, event: Event):
+        """Emit an event on the event bus. Only listener functions are notified
+
+        :param event: the event instance to emit
+        """
+        for listener in self._get_listeners_for_event(event):
+            try:
+                if asyncio.iscoroutinefunction(listener):
+                    logger.warning(
+                        "attempted to emit synchronous event to an async listener : %r",
+                        listener
+                    )
+                else:
+                    listener(event)
+
+            except Exception:
+                logger.exception(
+                    "exception notifying listener %r of event %r",
+                    listener, event
+                )
+
+    def _get_listeners_for_event(self, event: Event) -> list[EventListener]:
         try:
-            listeners = self._events[event.__class__]
+            listeners_refs = self._events[event.__class__]
         except KeyError:
-            pass
-        else:
-            for _, listener in listeners:
-                try:
-                    if asyncio.iscoroutinefunction(listener):
-                        await listener(event)
-                    else:
-                        listener(event)
-                except Exception:
-                    logger.exception(f"exception notifying listener {listener!r} of event {event!r}")
+            return []
+
+        listeners = []
+        for _, listener_ref in listeners_refs:
+            listener = listener_ref()
+            # Should never be None, but checked for type compatibility
+            if listener:  # pragma: no cover
+                listeners.append(listener)
+
+        return listeners
+
+    def _remove_callback(
+            self, event_class: type[E],
+            listener_ref: weakref.ReferenceType[EventListener]):
+
+        self._events[event_class] = [
+            listener_def
+            for listener_def in self._events[event_class]
+            if listener_ref != listener_def[1]
+        ]
 
 
 # Public events
@@ -180,7 +247,7 @@ class UserTrackingEvent(Event):
     """Emitted when a user is now successfully by the library tracked. This will
     be triggered when:
 
-    The `user` object will contain the tracked user. `data` will contain the
+    The ``user`` object will contain the tracked user. ``data`` will contain the
     data returned by the server
     """
     user: User
@@ -203,8 +270,6 @@ class UserUntrackingEvent(Event):
 class UserStatusUpdateEvent(Event):
     """Emitted when the server sends an update for the users status / privileges
     or a request for it
-
-    Following
     """
     before: User
     current: User
@@ -230,7 +295,7 @@ class UserInfoUpdateEvent(Event):
 
 @dataclass(frozen=True)
 class RoomListEvent(Event):
-    rooms: List[Room]
+    rooms: list[Room]
     raw_message: RoomList.Response
 
 
@@ -244,7 +309,7 @@ class RoomMessageEvent(Event):
 class RoomTickersEvent(Event):
     """Emitted when a list of tickers has been received for a room"""
     room: Room
-    tickers: Dict[str, str]
+    tickers: dict[str, str]
     raw_message: RoomTickers.Response
 
 
@@ -269,7 +334,8 @@ class RoomTickerRemovedEvent(Event):
 class RoomJoinedEvent(Event):
     """Emitted after a user joined a chat room
 
-    The value of `user` will be `None` in case it is us who has joined the room
+    The value of ``user`` will be ``None`` in case it is us who has joined the
+    room
     """
     room: Room
     raw_message: Union[JoinRoom.Response, UserJoinedRoom.Response]
@@ -280,7 +346,8 @@ class RoomJoinedEvent(Event):
 class RoomLeftEvent(Event):
     """Emitted after a user left a chat room
 
-    The value of `user` will be `None` in case it is us who has left the room
+    The value of ``user`` will be ``None`` in case it is us who has left the
+    room
     """
     room: Room
     raw_message: Union[LeaveRoom.Response, UserLeftRoom.Response]
@@ -291,8 +358,8 @@ class RoomLeftEvent(Event):
 class RoomMembershipGrantedEvent(Event):
     """Emitted when a member has been added to the private room
 
-    The value of `member` will be `None` in case it is us who has been added to
-    the room
+    The value of ``member`` will be ``None`` in case it is us who has been added
+    to the room
     """
     room: Room
     raw_message: Union[PrivateRoomGrantMembership.Response, PrivateRoomMembershipGranted.Response]
@@ -303,8 +370,8 @@ class RoomMembershipGrantedEvent(Event):
 class RoomMembershipRevokedEvent(Event):
     """Emitted when a member has been removed to the private room
 
-    The value of `member` will be `None` in case it is us who has been removed
-    from the room
+    The value of ``member`` will be ``None`` in case it is us who has been
+    removed from the room
     """
     room: Room
     raw_message: Union[PrivateRoomRevokeMembership.Response, PrivateRoomMembershipRevoked.Response]
@@ -316,8 +383,8 @@ class RoomOperatorGrantedEvent(Event):
     """Emitted when a member has been granted operator privileges on a private
     room
 
-    The value of `member` will be `None` in case it is us who has been granted
-    operator
+    The value of ``member`` will be ``None`` in case it is us who has been
+    granted operator
     """
     room: Room
     raw_message: Union[PrivateRoomGrantOperator.Response, PrivateRoomOperatorGranted.Response]
@@ -328,7 +395,7 @@ class RoomOperatorGrantedEvent(Event):
 class RoomOperatorRevokedEvent(Event):
     """Emitted when a member had operator privileges revoked on a private room
 
-    The value of `user` will be `None` in case it is us who has been revoked
+    The value of ``user`` will be ``None`` in case it is us who has been revoked
     operator
     """
     room: Room
@@ -340,7 +407,7 @@ class RoomOperatorRevokedEvent(Event):
 class RoomOperatorsEvent(Event):
     """Emitted when the server sends us a list of operators in a room"""
     room: Room
-    operators: List[User]
+    operators: list[User]
     raw_message: PrivateRoomOperators.Response
 
 
@@ -350,7 +417,7 @@ class RoomMembersEvent(Event):
     of members always excludes the owner of the room
     """
     room: Room
-    members: List[User]
+    members: list[User]
     raw_message: PrivateRoomMembers.Response
 
 
@@ -373,6 +440,18 @@ class PublicMessageEvent(Event):
     raw_message: PublicChatMessage.Response
 
 
+@dataclass(frozen=True)
+class SearchRequestSentEvent(Event):
+    """Emitted when a search request has been sent out"""
+    query: SearchRequest
+
+
+@dataclass(frozen=True)
+class SearchRequestRemovedEvent(Event):
+    """Emitted when a search request has been removed"""
+    query: SearchRequest
+
+
 @dataclass(frozen=True)
 class SearchResultEvent(Event):
     """Emitted when a search result has been received"""
@@ -390,50 +469,50 @@ class SearchRequestReceivedEvent(Event):
 
 @dataclass(frozen=True)
 class SimilarUsersEvent(Event):
-    users: List[Tuple[User, int]]
+    users: list[tuple[User, int]]
     raw_message: GetSimilarUsers.Response
 
 
 @dataclass(frozen=True)
 class ItemSimilarUsersEvent(Event):
     item: str
-    users: List[User]
+    users: list[User]
     raw_message: GetItemSimilarUsers.Response
 
 
 @dataclass(frozen=True)
 class RecommendationsEvent(Event):
-    recommendations: List[Recommendation]
-    unrecommendations: List[Recommendation]
+    recommendations: list[Recommendation]
+    unrecommendations: list[Recommendation]
     raw_message: GetRecommendations.Response
 
 
 @dataclass(frozen=True)
 class GlobalRecommendationsEvent(Event):
-    recommendations: List[Recommendation]
-    unrecommendations: List[Recommendation]
+    recommendations: list[Recommendation]
+    unrecommendations: list[Recommendation]
     raw_message: GetGlobalRecommendations.Response
 
 
 @dataclass(frozen=True)
 class ItemRecommendationsEvent(Event):
     item: str
-    recommendations: List[Recommendation]
+    recommendations: list[Recommendation]
     raw_message: GetItemRecommendations.Response
 
 
 @dataclass(frozen=True)
 class UserInterestsEvent(Event):
     user: User
-    interests: List[str]
-    hated_interests: List[str]
+    interests: list[str]
+    hated_interests: list[str]
     raw_message: GetUserInterests.Response
 
 
 @dataclass(frozen=True)
 class PrivilegedUsersEvent(Event):
     """Emitted when the list of privileged users has been received"""
-    users: List[User]
+    users: list[User]
     raw_message: PrivilegedUsers.Response
 
 
@@ -446,7 +525,7 @@ class PrivilegedUserAddedEvent(Event):
 @dataclass(frozen=True)
 class PrivilegesUpdateEvent(Event):
     """Emitted when we receive a message containing how much time is left on the
-    current user's privileges. If `time_left` is 0 the current user has no
+    current user's privileges. If ``time_left`` is 0 the current user has no
     privileges
     """
     time_left: int
@@ -458,8 +537,8 @@ class PrivilegesUpdateEvent(Event):
 @dataclass(frozen=True)
 class UserSharesReplyEvent(Event):
     user: User
-    directories: List[DirectoryData]
-    locked_directories: List[DirectoryData]
+    directories: list[DirectoryData]
+    locked_directories: list[DirectoryData]
     raw_message: PeerSharesReply.Request
 
 
@@ -467,7 +546,7 @@ class UserSharesReplyEvent(Event):
 class UserDirectoryEvent(Event):
     user: User
     directory: str
-    directories: List[DirectoryData]
+    directories: list[DirectoryData]
     raw_message: PeerDirectoryContentsReply.Request
 
 
@@ -488,13 +567,13 @@ class TransferRemovedEvent(Event):
 @dataclass(frozen=True)
 class TransferProgressEvent(Event):
     """Called periodically to report progress. The interval is determined by the
-    `transfers.progress_interval` settings parameter
+    ``transfers.progress_interval`` settings parameter
 
     This event only includes transfers where the transfer state has been changed
     since the previous event. If there are no updates the event will no be
     called
     """
-    updates: List[Tuple[Transfer, TransferProgressSnapshot, TransferProgressSnapshot]]
+    updates: list[tuple[Transfer, TransferProgressSnapshot, TransferProgressSnapshot]]
     """List of progress updates: transfer instance, previous snapshot, current
     snapshot
     """
@@ -545,3 +624,24 @@ class ScanCompleteEvent(InternalEvent):
     """Emitted when shares scan was completed"""
     folder_count: int
     file_count: int
+
+
+@dataclass(frozen=True)
+class FriendListChangedEvent(InternalEvent):
+    """Emitted when a change was detected in the friends list"""
+    added: set[str] = field(default_factory=set)
+    removed: set[str] = field(default_factory=set)
+
+
+@dataclass(frozen=True)
+class BlockListChangedEvent(InternalEvent):
+    """Emitted when a change was detected in the blocked list"""
+    changes: dict[str, tuple[BlockingFlag, BlockingFlag]]
+    """List of changes. The key contains the username, the value is a tuple
+    containing the old and new flags.
+    """
+
+
+@dataclass
+class SharedDirectoryChangeEvent(InternalEvent):
+    shared_directory: SharedDirectory
