@@ -1,18 +1,22 @@
 from __future__ import annotations
+import abc
 from aiofiles.threadpool.binary import AsyncBufferedIOBase, AsyncBufferedReader
 import asyncio
-from async_timeout import timeout as atimeout
+from async_timeout import Timeout, timeout as atimeout
+from collections.abc import Awaitable, Callable
 from enum import auto, Enum
-from typing import Callable, List, Optional, TYPE_CHECKING, Union
+from typing import Optional, TYPE_CHECKING, Union
 import logging
 import socket
 import struct
 
 from ..constants import (
+    DEFAULT_READ_TIMEOUT,
     DISCONNECT_TIMEOUT,
     PEER_CONNECT_TIMEOUT,
     PEER_READ_TIMEOUT,
     SERVER_CONNECT_TIMEOUT,
+    SERVER_READ_TIMEOUT,
     TRANSFER_TIMEOUT,
 )
 from ..exceptions import (
@@ -30,6 +34,7 @@ from ..protocol.messages import (
     PeerMessage,
     ServerMessage,
 )
+from ..log_utils import ConnectionLoggerAdapter
 from ..utils import task_counter
 from .rate_limiter import RateLimiter, UnlimitedRateLimiter
 
@@ -38,13 +43,9 @@ if TYPE_CHECKING:
 
 
 logger = logging.getLogger(__name__)
+adapter = ConnectionLoggerAdapter(logger, {})
+
 
-DEFAULT_RECV_BUF_SIZE = 8
-"""Default amount of bytes to recv from the socket"""
-TRANSFER_RECV_BUF_SIZE = 1024 * 8
-"""Default amount of bytes to recv during file transfer"""
-TRANSFER_SEND_BUF_SIZE = 1024 * 8
-"""Default amount of bytes to send during file transfer"""
 HEADER_SIZE_OBFUSCATED: int = obfuscation.KEY_SIZE + struct.calcsize('I')
 HEADER_SIZE_UNOBFUSCATED: int = struct.calcsize('I')
 
@@ -86,24 +87,27 @@ class PeerConnectionState(Enum):
     """Transfer connections: ready to receive / send data"""
 
 
-class Connection:
+class Connection(abc.ABC):
+
+    _CLOSING_STATES: tuple[ConnectionState, ConnectionState] = (
+        ConnectionState.CLOSING, ConnectionState.CLOSED)
 
     def __init__(self, hostname: str, port: int, network: Network):
         self.hostname: str = hostname
         self.port: int = port
         self.network: Network = network
         self.state: ConnectionState = ConnectionState.UNINITIALIZED
+        self._is_closing: bool = False
 
+    @abc.abstractmethod
     async def disconnect(self, reason: CloseReason = CloseReason.UNKNOWN):
-        raise NotImplementedError()
+        ...
 
     async def set_state(self, state: ConnectionState, close_reason: CloseReason = CloseReason.UNKNOWN):
         self.state = state
+        self._is_closing = state in self._CLOSING_STATES
         await self.network.on_state_changed(state, self, close_reason=close_reason)
 
-    def _is_closing(self) -> bool:
-        return self.state in (ConnectionState.CLOSING, ConnectionState.CLOSED)
-
     def __repr__(self) -> str:
         return (
             f"{self.__class__.__name__}(hostname={self.hostname!r}, port={self.port}, "
@@ -120,33 +124,34 @@ class ListeningConnection(Connection):
         self.obfuscated: bool = obfuscated
 
         self._server: Optional[asyncio.AbstractServer] = None
-        self.connections_accepted: int = 0
 
     async def disconnect(self, reason: CloseReason = CloseReason.UNKNOWN):
-        logger.debug(f"{self.hostname}:{self.port} : disconnecting : {reason.name}")
+        adapter.debug("disconnecting : %s", reason.name, extra=self.__dict__)
         await self.set_state(ConnectionState.CLOSING, close_reason=reason)
         try:
             if self._server is not None:
                 if self._server.is_serving():
                     self._server.close()
-                await self._server.wait_closed()
+
+                async with atimeout(DISCONNECT_TIMEOUT):
+                    await self._server.wait_closed()
 
         except Exception as exc:
-            logger.warning(
-                f"{self.hostname}:{self.port} : exception while disconnecting", exc_info=exc)
+            adapter.warning(
+                "exception while disconnecting", extra=self.__dict__, exc_info=exc)
 
         finally:
             self._server = None
-            logger.debug(f"{self.hostname}:{self.port} : disconnected : {reason.name}")
+            adapter.info("disconnected : %s", reason.name, extra=self.__dict__)
             await self.set_state(ConnectionState.CLOSED, close_reason=reason)
 
     async def connect(self):
-        """Open a listening connection on the current `hostname` and `port`
+        """Open a listening connection on the current :attr:`hostname` and
+        :attr:`port`
 
         :raise ConnectionFailedError: raised when binding failed
         """
-        logger.info(
-            f"open {self.hostname}:{self.port} : listening connection")
+        adapter.info("open listening connection", extra=self.__dict__)
         await self.set_state(ConnectionState.CONNECTING)
 
         try:
@@ -166,7 +171,10 @@ class ListeningConnection(Connection):
 
     async def accept(self, reader: asyncio.StreamReader, writer: asyncio.StreamWriter):
         hostname, port = writer.get_extra_info('peername')
-        logger.debug(f"{self.hostname}:{self.port} : accepted connection {hostname}:{port}")
+
+        adapter.debug(
+            "accepted connection : %s:%d", hostname, port, extra=self.__dict__)
+
         connection = PeerConnection(
             hostname, port, self.network,
             obfuscated=self.obfuscated,
@@ -178,18 +186,24 @@ class ListeningConnection(Connection):
         await connection.set_state(ConnectionState.CONNECTED)
 
 
-class DataConnection(Connection):
+class DataConnection(Connection, abc.ABC):
     """Connection for message and data transfer"""
 
-    def __init__(self, hostname: str, port: int, network: Network, obfuscated: bool = False):
+    def __init__(
+            self, hostname: str, port: int, network: Network,
+            obfuscated: bool = False, read_timeout: float = DEFAULT_READ_TIMEOUT):
+
         super().__init__(hostname, port, network)
         self.obfuscated = obfuscated
 
         self._reader: Optional[asyncio.StreamReader] = None
         self._writer: Optional[asyncio.StreamWriter] = None
         self._reader_task: Optional[asyncio.Task] = None
-        self.read_timeout: Optional[float] = None
-        self._queued_messages: List[asyncio.Task] = []
+
+        self._queued_messages: list[asyncio.Task] = []
+        self._read_timeout_object: Optional[Timeout] = None
+
+        self.read_timeout: float = read_timeout
 
     def get_connecting_ip(self) -> str:
         """Gets the IP address being used to connect to the server/peer.
@@ -203,17 +217,17 @@ class DataConnection(Connection):
 
     async def connect(self, timeout: float = 30):
         """Opens a TCP connection the hostname:ip provided in this object. The
-        state of the connection will be changed to CONNECTING before the attempt
-        is made
+        state of the connection will be changed to ``CONNECTING`` before the
+        attempt is made
 
-        Upon success the `state` will be set to CONNECTED and the message
-        reader loop will be started. Upon failure the state will be set to
-        CLOSED with CONNECT_FAILED as its reason
+        Upon success the :attr:`state` will be set to ``CONNECTED`` and the
+        message reader loop will be started. Upon failure the state will be set
+        to ``CLOSED`` with ``CONNECT_FAILED`` as its reason
 
         :param timeout: timeout in seconds before giving up (default: 30)
         :raise ConnectionFailedError: raised when connection failed or timed out
         """
-        logger.info(f"{self.hostname}:{self.port} : connecting")
+        adapter.info("connecting", extra=self.__dict__)
         await self.set_state(ConnectionState.CONNECTING)
 
         try:
@@ -226,7 +240,7 @@ class DataConnection(Connection):
             raise ConnectionFailedError(f"{self.hostname}:{self.port} : failed to connect") from exc
 
         else:
-            logger.debug(f"{self.hostname}:{self.port} : connected")
+            adapter.debug("connected", extra=self.__dict__)
             await self.set_state(ConnectionState.CONNECTED)
 
     async def disconnect(self, reason: CloseReason = CloseReason.UNKNOWN):
@@ -243,7 +257,7 @@ class DataConnection(Connection):
             return
 
         await self.set_state(ConnectionState.CLOSING, close_reason=reason)
-        logger.debug(f"{self.hostname}:{self.port} : disconnecting : {reason.name}")
+        adapter.debug("disconnecting : %s", reason.name, extra=self.__dict__)
         self._cancel_queued_messages()
         try:
             if self._writer is not None:
@@ -254,7 +268,8 @@ class DataConnection(Connection):
                     await self._writer.wait_closed()
 
         except Exception as exc:
-            logger.warning(f"{self.hostname}:{self.port} : exception while disconnecting : {exc!r}")
+            adapter.warning(
+                "exception while disconnecting : %r", exc, extra=self.__dict__)
 
         finally:
             await self.set_state(ConnectionState.CLOSED, close_reason=reason)
@@ -265,140 +280,67 @@ class DataConnection(Connection):
             self._writer = None
 
     def start_reader_task(self):
+        """Starts the message reader task"""
         self._reader_task = asyncio.create_task(self._message_reader_loop())
 
     def stop_reader_task(self):
+        """Stops the message reader task if it exists"""
         if self._reader_task is not None:
             self._reader_task.cancel()
             self._reader_task = None
 
     # Read/write methods
 
-    async def _read_until_eof(self) -> bytes:
-        return await self._reader.read(-1)  # type: ignore[union-attr]
-
-    async def receive_until_eof(self, raise_exception: bool = True) -> Optional[bytes]:
-        if not self._reader:
-            raise ConnectionReadError("cannot read until EOF, connection is not open")
-
-        try:
-            return await self._read(self._read_until_eof)
-        except ConnectionReadError:
-            if raise_exception:
-                raise
-
-            return None
-
     async def _message_reader_loop(self):
         """Message reader loop. This will loop until the connection is closed or
         the network is closed
         """
-        while not self._is_closing():
+        while not self._is_closing:
             try:
-                message_data = await self.receive_message()
-                if message_data:
-                    message = self.decode_message_data(message_data)
+                message = await self.receive_message_object()
 
             except ConnectionReadError:
-                logger.warning(f"{self.hostname}:{self.port} : read error")
+                adapter.warning("read error", extra=self.__dict__)
 
             except MessageDeserializationError:
-                logger.warning(f"{self.hostname}:{self.port} : failed to deserialize message")
+                adapter.warning("failed to deserialize message : %s", message, extra=self.__dict__)
 
             else:
+
+                if not message:
+                    return
+
                 # Do not handle messages when closing/closed
-                if not self._is_closing():
-                    if message:
-                        await self._perform_message_callback(message)
+                if self._is_closing:
+                    adapter.warning(
+                        "connection is closing, skipping handling message : %s",
+                        message,
+                        extra=self.__dict__
+                    )
+                else:
+                    await self._perform_message_callback(message)
 
-    async def _read_message(self) -> bytes:
-        if not self._reader:
-            raise ConnectionReadError("cannot read message, connection is not open")
+    async def _read(
+            self, reader_coro: Awaitable[Optional[bytes]],
+            timeout: Optional[float] = None) -> Optional[bytes]:
+        """Read data from the connection using the passed ``reader_coro``. When
+        an error occurs during reading the connection will be CLOSED
 
-        header_size = HEADER_SIZE_OBFUSCATED if self.obfuscated else HEADER_SIZE_UNOBFUSCATED
-
-        async with atimeout(self.read_timeout):
-            header = await self._reader.readexactly(header_size)
-
-        message_len_buf = obfuscation.decode(header) if self.obfuscated else header
-        _, message_len = uint32.deserialize(0, message_len_buf)
-
-        async with atimeout(self.read_timeout):
-            message = await self._reader.readexactly(message_len)  #
-
-        return header + message
-
-    async def receive_message_object(self) -> Optional[MessageDataclass]:
-        message_data = await self.receive_message()
-        if message_data:
-            return self.decode_message_data(message_data)
-        else:
-            return None
-
-    async def receive_message(self) -> Optional[bytes]:
-        """Receives a single raw message"""
-        return await self._read(self._read_message)
-
-    def decode_message_data(self, data: bytes) -> MessageDataclass:
-        """De-obfuscates and deserializes message data into a `MessageDataclass`
-        object. See `deserialize_message`
-
-        :param data: message bytes to decode
-        :return: object of `MessageDataclass`
-        :raise MessageDeserializationError: raised when deserialization failed
-        """
-        if self.obfuscated:
-            data = obfuscation.decode(data)
-
-        try:
-            message = self.deserialize_message(data)
-        except Exception as exc:
-            logger.exception(f"{self.hostname}:{self.port} : failed to deserialize message : {data.hex()}")
-            raise MessageDeserializationError("failed to deserialize message") from exc
-
-        return message
-
-    def encode_message_data(self, message: Union[bytes, MessageDataclass]) -> bytes:
-        """Serializes the `MessageDataclass` or `bytes` and obfuscates the
-        contents. See `serialize_message`
-
-        :return: `bytes` object
-        :raise MessageSerializationError: raised when serialization failed
-        """
-        try:
-            data = self.serialize_message(message)
-        except Exception as exc:
-            logger.exception(f"{self.hostname}:{self.port} : failed to serialize message : {message!r}")
-            raise MessageSerializationError("failed to serialize Message") from exc
-
-        if self.obfuscated:
-            data = obfuscation.encode(data)
-
-        return data
-
-    async def _perform_message_callback(self, message: MessageDataclass):
-        logger.debug(f"{self.hostname}:{self.port} : received message : {message!r}")
-        try:
-            await self.network.on_message_received(message, self)
-        except Exception:
-            logger.exception(f"error during callback : {message!r}")
-
-    async def _read(self, reader_func, timeout: Optional[float] = None) -> Optional[bytes]:
-        """Read data from the connection using the passed `reader_func`. When an
-        error occurs during reading the connection will be CLOSED
-
-        :param reader_func: callable that reads the data
-        :return: the return value of the `reader_func`, None if EOF
+        :param reader_coro: coroutine that reads the data
+        :return: the return value of the ``reader_coro``, ``None`` if EOF
         :raise ConnectionReadError: upon any kind of read error/timeout
         """
         try:
             if timeout:
-                async with atimeout(timeout):
-                    return await reader_func()
+                # TODO: change the type of `_read_timeout_object` to `asyncio.timeouts.Timeout`
+                # when 3.11 is the oldest supported Python version
+                async with atimeout(timeout) as self._read_timeout_object:  # type: ignore[assignment]
+                    data = await reader_coro
             else:
-                return await reader_func()
+                data = await reader_coro
 
         except asyncio.IncompleteReadError as exc:
+            # Can only be raised by `readexactly`
             if exc.partial:
                 await self.disconnect(CloseReason.READ_ERROR)
                 raise ConnectionReadError(
@@ -415,6 +357,79 @@ class DataConnection(Connection):
             await self.disconnect(CloseReason.READ_ERROR)
             raise ConnectionReadError(f"{self.hostname}:{self.port} : exception during reading") from exc
 
+        else:
+            if not data:
+                await self.disconnect(CloseReason.EOF)
+                return None
+            else:
+                return data
+
+    async def _read_message(self) -> bytes:
+        if not self._reader:
+            raise ConnectionReadError(
+                f"{self.hostname}:{self.port} : cannot read message, connection is not open")
+
+        header_size = HEADER_SIZE_OBFUSCATED if self.obfuscated else HEADER_SIZE_UNOBFUSCATED
+
+        header = await self._reader.readexactly(header_size)
+
+        message_len_buf = obfuscation.decode(header) if self.obfuscated else header
+        _, message_len = uint32.deserialize(0, message_len_buf)
+
+        message = await self._reader.readexactly(message_len)
+
+        return header + message
+
+    async def receive_message(self) -> Optional[bytes]:
+        """Receives a single raw message
+
+        :return: The message as a ``bytes`` object. ``None`` if the connection
+            returned EOF
+        """
+        return await self._read(self._read_message(), timeout=self.read_timeout)
+
+    async def receive_message_object(self) -> Optional[MessageDataclass]:
+        """Receives a single message and parses it to a :class:`.MessageDataclass`
+
+        :return: The parsed message, ``None`` if the connection returned EOF
+        """
+        message_data = await self.receive_message()
+
+        if message_data:
+            message = self.decode_message_data(message_data)
+            adapter.debug(
+                "received message : %s",
+                message,
+                extra={
+                    **self.__dict__,
+                    **{'message_type': message.__class__}
+                }
+            )
+            return message
+
+        return None
+
+    async def receive_until_eof(self, raise_exception: bool = True) -> Optional[bytes]:
+        """Receives data until the other end closes the connection. If
+        ``raise_exception`` parameter is set to ``True`` other read errors are
+        treated as EOF as well
+        """
+        if not self._reader:
+            raise ConnectionReadError(
+                f"{self.hostname}:{self.port} : cannot read until EOF, connection is not open")
+
+        # Note: _read raises ConnectionReadError also on IncompleteReadError,
+        # however when using StreamReader.read this error can never be raised
+        # and thus this method shouldn't worry about potentially not returning
+        # partial data
+        try:
+            return await self._read(self._reader.read(-1))
+        except ConnectionReadError:
+            if raise_exception:
+                raise
+
+            return b''
+
     def queue_message(self, message: Union[bytes, MessageDataclass]) -> asyncio.Task:
         task = asyncio.create_task(
             self.send_message(message),
@@ -424,44 +439,23 @@ class DataConnection(Connection):
         task.add_done_callback(self._queued_messages.remove)
         return task
 
-    def queue_messages(self, *messages: Union[bytes, MessageDataclass]) -> List[asyncio.Task]:
+    def queue_messages(self, *messages: Union[bytes, MessageDataclass]) -> list[asyncio.Task]:
         return [
             self.queue_message(message)
             for message in messages
         ]
 
-    def _cancel_queued_messages(self):
-        for qmessage_task in self._queued_messages:
-            qmessage_task.cancel()
-
-    async def send_message(self, message: Union[bytes, MessageDataclass]):
-        """Sends a message or a set of bytes over the connection. In case an
-        object of `MessageDataClass` is provided the object will first be
-        serialized. If the `obfuscated` flag is set for the connection the
-        message or bytes will first be obfuscated
-
-        :param message: message to be sent over the connection
-        :raise ConnectionWriteError: error or timeout occured during writing
-        """
-        if self._is_closing():
-            logger.warning(
-                f"{self.hostname}:{self.port} : not sending message, connection is closing / closed : {message!r}")
-            return
-
+    async def _send(self, data: bytes, timeout: Optional[float] = None):
         if not self._writer:
-            raise ConnectionWriteError("cannot send message, connection is not open")
+            raise ConnectionWriteError(
+                f"{self.hostname}:{self.port} : cannot send data, connection is not open")
 
-        logger.debug(f"{self.hostname}:{self.port} : send message : {message!r}")
-        # Serialize the message
-        try:
-            data = self.encode_message_data(message)
-        except MessageSerializationError:
-            return
-
-        # Perform actual send
         try:
             self._writer.write(data)
-            async with atimeout(10):
+            if timeout:
+                async with atimeout(timeout):
+                    await self._writer.drain()
+            else:
                 await self._writer.drain()
 
         except asyncio.TimeoutError as exc:
@@ -472,12 +466,81 @@ class DataConnection(Connection):
             await self.disconnect(CloseReason.WRITE_ERROR)
             raise ConnectionWriteError(f"{self.hostname}:{self.port} : exception during writing") from exc
 
+    async def send_message(self, message: Union[bytes, MessageDataclass]):
+        """Sends a message or a set of bytes over the connection. In case an
+        object of :class:`.MessageDataClass` is provided the object will first
+        be serialized. If the :attr:`obfuscated` flag is set for the connection
+        the message or bytes will first be obfuscated
+
+        :param message: message to be sent over the connection
+        :raise ConnectionWriteError: error or timeout occured during writing
+        """
+        if self._is_closing:
+            adapter.warning(
+                "not sending message, connection is closing / closed : %s",
+                message,
+                extra=self.__dict__
+            )
+            return
+
+        adapter.debug(
+            "send message : %s",
+            message,
+            extra={
+                **self.__dict__,
+                **{'message_type': message.__class__}
+            }
+        )
+        # Serialize the message
+        try:
+            data = self.encode_message_data(message)
+        except MessageSerializationError:
+            adapter.exception("failed to serialize message : %s", message)
+            return
+
+        await self._send(data, timeout=10)
+        self._increase_read_timeout()
+
+    def encode_message_data(self, message: Union[bytes, MessageDataclass]) -> bytes:
+        """Serializes the :class:`.MessageDataclass` or ``bytes`` and obfuscates
+        the contents. See :meth:`serialize_message`
+
+        :return: ``bytes`` object
+        :raise MessageSerializationError: raised when serialization failed
+        """
+        try:
+            data = self.serialize_message(message)
+        except Exception as exc:
+            raise MessageSerializationError("failed to serialize message") from exc
+
+        if self.obfuscated:
+            data = obfuscation.encode(data)
+
+        return data
+
+    def decode_message_data(self, data: bytes) -> MessageDataclass:
+        """De-obfuscates and deserializes message data into a
+        :class:`.MessageDataclass` object. See :meth:`deserialize_message`
+
+        :param data: message bytes to decode
+        :return: object of :class:`.MessageDataclass`
+        :raise MessageDeserializationError: raised when deserialization failed
+        """
+        if self.obfuscated:
+            data = obfuscation.decode(data)
+
+        try:
+            message = self.deserialize_message(data)
+        except Exception as exc:
+            raise MessageDeserializationError("failed to deserialize message") from exc
+
+        return message
+
+    @abc.abstractmethod
     def deserialize_message(self, message_data: bytes) -> MessageDataclass:
         """Should be called after a full message has been received. This method
         should parse the message
         """
-        raise NotImplementedError(
-            "'deserialize_message' should be overwritten by a subclass")
 
     def serialize_message(self, message: Union[bytes, MessageDataclass]) -> bytes:
         if isinstance(message, MessageDataclass):
@@ -485,12 +548,36 @@ class DataConnection(Connection):
         else:
             return message
 
+    def _increase_read_timeout(self):
+        if self.read_timeout and self._read_timeout_object:
+            try:
+                self._read_timeout_object.shift(self.read_timeout)
+            except RuntimeError:
+                pass
+
+    async def _perform_message_callback(self, message: MessageDataclass):
+        try:
+            await self.network.on_message_received(message, self)
+        except Exception:
+            adapter.exception("error during callback : %s", message, extra=self.__dict__)
+
+    def _cancel_queued_messages(self):
+        for qmessage_task in self._queued_messages:
+            qmessage_task.cancel()
+
 
 class ServerConnection(DataConnection):
 
+    def __init__(
+            self, hostname: str, port: int, network: Network,
+            obfuscated: bool = False, read_timeout: float = SERVER_READ_TIMEOUT):
+
+        super().__init__(
+            hostname, port, network,
+            obfuscated=obfuscated, read_timeout=read_timeout)
+
     async def connect(self, timeout: float = SERVER_CONNECT_TIMEOUT):
         await super().connect(timeout=timeout)
-        # self.start_reader_task()
 
     def deserialize_message(self, message_data: bytes) -> MessageDataclass:
         return ServerMessage.deserialize_response(message_data)
@@ -501,14 +588,20 @@ class PeerConnection(DataConnection):
     def __init__(
             self, hostname: str, port: int, network: Network, obfuscated: bool = False,
             username: Optional[str] = None, connection_type: str = PeerConnectionType.PEER,
-            incoming: bool = False):
-        super().__init__(hostname, port, network, obfuscated=obfuscated)
+            incoming: bool = False, read_timeout: float = PEER_READ_TIMEOUT,
+            transfer_read_timeout: float = TRANSFER_TIMEOUT):
+
+        super().__init__(
+            hostname, port, network,
+            obfuscated=obfuscated, read_timeout=read_timeout)
+
+        self.transfer_read_timeout: float = transfer_read_timeout
+
         self.incoming: bool = incoming
         self.connection_state = PeerConnectionState.AWAITING_INIT
+        self.connection_type: str = connection_type
 
         self.username: Optional[str] = username
-        self.connection_type: str = connection_type
-        self.read_timeout = PEER_READ_TIMEOUT
 
         self.download_rate_limiter: RateLimiter = UnlimitedRateLimiter()
         self.upload_rate_limiter: RateLimiter = UnlimitedRateLimiter()
@@ -519,13 +612,13 @@ class PeerConnection(DataConnection):
     def set_connection_state(self, state: PeerConnectionState):
         """Sets the current connection state.
 
-        If the current state is AWAITING_INIT and the connection type is a
-        distributed or file connection the `obfuscated` flag for this connection
-        will be set to `False`
+        If the current state is ``AWAITING_INIT`` and the connection type is a
+        distributed or file connection the :attr:`obfuscated` flag for this
+        connection will be set to ``False``
 
-        If the state goes to the ESTABLISHED state the message reader task will
-        be started. In all other cases it will be stopped (in case the task was
-        running)
+        If the state goes to the ``ESTABLISHED`` state the message reader task
+        will be started. In all other cases it will be stopped (in case the
+        task was running)
 
         :param state: The new state of the connection
         """
@@ -541,84 +634,75 @@ class PeerConnection(DataConnection):
             # receive_* methods directly. But it can do no harm
             self.stop_reader_task()
 
-        logger.debug(f"{self.hostname}:{self.port} setting state to {state} : {self!r}")
+        adapter.debug("setting state to %s", state, extra=self.__dict__)
         self.connection_state = state
 
-    async def _read_transfer_ticket(self) -> bytes:
-        if not self._reader:
-            raise ConnectionReadError("cannot read transfer ticket, connection is not open")
-
-        return await self._reader.readexactly(struct.calcsize('I'))
-
-    async def _read_transfer_offset(self) -> bytes:
-        if not self._reader:
-            raise ConnectionReadError("cannot read transfer offset, connection is not open")
-
-        return await self._reader.readexactly(struct.calcsize('Q'))
-
     async def receive_transfer_ticket(self) -> int:
         """Receive the transfer ticket from the connection"""
-        data = await self._read(self._read_transfer_ticket)
+        if not self._reader:
+            raise ConnectionReadError(
+                f"{self.hostname}:{self.port} : cannot read transfer ticket, "
+                "connection is not open"
+            )
 
+        data = await self._read(self._reader.readexactly(struct.calcsize('I')))
         if data is None:  # pragma: no cover
             raise ConnectionReadError(
-                "couldn't receive transfer ticket, connection closed")
+                f"{self.hostname}:{self.port} : couldn't receive transfer ticket, "
+                "connection closed"
+            )
 
         _, ticket = uint32.deserialize(0, data)
         return ticket
 
     async def receive_transfer_offset(self) -> int:
         """Receive the transfer offset from the connection"""
-        data = await self._read(self._read_transfer_offset)
+        if not self._reader:
+            raise ConnectionReadError(
+                f"{self.hostname}:{self.port} : cannot read transfer offset, "
+                "connection is not open"
+            )
 
+        data = await self._read(self._reader.readexactly(struct.calcsize('Q')))
         if data is None:  # pragma: no cover
             raise ConnectionReadError(
-                "couldn't receive transfer offset, connection closed")
+                f"{self.hostname}:{self.port} : couldn't receive transfer offset, "
+                "connection closed"
+            )
 
         _, offset = uint64.deserialize(0, data)
         return offset
 
-    async def receive_data(self, n_bytes: int, timeout: float = TRANSFER_TIMEOUT) -> Optional[bytes]:
-        """Receives given amount of bytes on the connection.
+    async def receive_data(self, n_bytes: int) -> Optional[bytes]:
+        """Receives data on the connection. The ``n_bytes`` indicates the max
+        amount of bytes that should be received on the connection, less bytes
+        can be returned.
 
         In case of error the socket will be disconnected. If no data is received
         it is assumed to be EOF and the connection will be disconnected.
 
-        :param n_bytes: amount of bytes to receive
-        :param timeout: timeout in seconds
+        :param n_bytes: max amount of bytes to receive
         :raise ConnectionReadError: in case timeout occured on an error on the
             socket
-        :return: `bytes` object containing the received data
+        :return: ``bytes`` object containing the received data or ``None`` in
+            case the connection reached EOF
         """
         if not self._reader:
-            raise ConnectionReadError("cannot read data, connection is not open")
+            raise ConnectionReadError(
+                f"{self.hostname}:{self.port} : cannot read data, connection is not open")
 
-        try:
-            async with atimeout(timeout):
-                data = await self._reader.read(n_bytes)
-
-        except asyncio.TimeoutError as exc:
-            await self.disconnect(CloseReason.TIMEOUT)
-            raise ConnectionReadError(f"{self.hostname}:{self.port} : timeout writing") from exc
-
-        except Exception as exc:
-            await self.disconnect(CloseReason.READ_ERROR)
-            raise ConnectionReadError(f"{self.hostname}:{self.port} : read error") from exc
-
-        else:
-            if not data:
-                await self.disconnect(CloseReason.EOF)
-                return None
-            else:
-                return data
+        return await self._read(
+            self._reader.read(n_bytes), timeout=self.transfer_read_timeout)
 
     async def receive_file(
-            self, file_handle: AsyncBufferedIOBase, filesize: int, callback: Optional[Callable[[bytes], None]] = None):
+            self, file_handle: AsyncBufferedIOBase, filesize: int,
+            callback: Optional[Callable[[bytes], None]] = None):
         """Receives a file on the current connection and writes it to the given
-        `file_handle`
+        ``file_handle``
 
         This method will attempt to keep reading data until EOF is received from
-        the connection or the amount of received bytes has reached the `filesize`
+        the connection or the amount of received bytes has reached the
+        ``filesize``
 
         :param file_handle: a file handle to write the received data to
         :param filesize: expected filesize
@@ -642,25 +726,12 @@ class PeerConnection(DataConnection):
                 return
 
     async def send_data(self, data: bytes):
-        if not self._writer:
-            raise ConnectionWriteError("cannot send data, connection is not open")
-
-        try:
-            self._writer.write(data)
-            async with atimeout(TRANSFER_TIMEOUT):
-                await self._writer.drain()
-
-        except asyncio.TimeoutError as exc:
-            await self.disconnect(CloseReason.TIMEOUT)
-            raise ConnectionWriteError(f"{self.hostname}:{self.port} : timeout writing") from exc
-
-        except Exception as exc:
-            await self.disconnect(CloseReason.WRITE_ERROR)
-            raise ConnectionWriteError(f"{self.hostname}:{self.port} : write error") from exc
+        await self._send(data, timeout=TRANSFER_TIMEOUT)
 
     async def send_file(self, file_handle: AsyncBufferedReader, callback: Optional[Callable[[bytes], None]] = None):
         """Sends a file over the connection. This method makes use of the
-        `upload_rate_limiter` to limit how many bytes are being sent at a time.
+        :attr:`upload_rate_limiter` to limit how many bytes are being sent at a
+        time
 
         :param file_handle: binary opened file handle
         :param callback: progress callback that gets called each time a chunk
