@@ -23,6 +23,7 @@ from .settings import Settings
 from .shares.manager import SharesManager
 from .transfer.interface import UploadInfoProvider
 from .user.manager import UserManager
+from .user.model import BlockingFlag
 from .utils import ticket_generator
 
 
@@ -38,6 +39,7 @@ class PeerManager(BaseManager):
             user_manager: UserManager,
             shares_manager: SharesManager, upload_info_provider: UploadInfoProvider,
             network: Network):
+
         self._settings: Settings = settings
         self._event_bus: EventBus = event_bus
         self._network: Network = network
@@ -59,11 +61,15 @@ class PeerManager(BaseManager):
 
     @on_message(PeerSharesRequest.Request)
     async def _on_peer_shares_request(self, message: PeerSharesRequest.Request, connection: PeerConnection):
+
         if not connection.username:
             logger.warning(
                 "got PeerSharesRequest for a connection that wasn't properly initialized")
             return
 
+        if self._settings.users.is_blocked(connection.username, BlockingFlag.SHARES):
+            return
+
         visible, locked = self._shares_manager.create_shares_reply(connection.username)
         await connection.send_message(
             PeerSharesReply.Request(
@@ -74,13 +80,16 @@ class PeerManager(BaseManager):
 
     @on_message(PeerSharesReply.Request)
     async def _on_peer_shares_reply(self, message: PeerSharesReply.Request, connection: PeerConnection):
+
         if not connection.username:
             logger.warning(
                 "got PeerSharesRequest for a connection that wasn't properly initialized")
             return
 
         logger.info(
-            f"PeerSharesReply : from username {connection.username}, got {len(message.directories)} directories")
+            "PeerSharesReply : from username %s, got %d directories",
+            connection.username, len(message.directories)
+        )
 
         user = self._user_manager.get_user_object(connection.username)
 
@@ -96,11 +105,15 @@ class PeerManager(BaseManager):
     @on_message(PeerDirectoryContentsRequest.Request)
     async def _on_peer_directory_contents_req(
             self, message: PeerDirectoryContentsRequest.Request, connection: PeerConnection):
+
         if not connection.username:
             logger.warning(
                 "got PeerDirectoryContentsRequest for a connection that wasn't properly initialized")
             return
 
+        if self._settings.users.is_blocked(connection.username, BlockingFlag.SHARES):
+            return
+
         directories = self._shares_manager.create_directory_reply(message.directory)
         await connection.send_message(
             PeerDirectoryContentsReply.Request(
@@ -113,6 +126,7 @@ class PeerManager(BaseManager):
     @on_message(PeerDirectoryContentsReply.Request)
     async def _on_peer_directory_contents_reply(
             self, message: PeerDirectoryContentsReply.Request, connection: PeerConnection):
+
         if not connection.username:
             logger.warning(
                 "got PeerDirectoryContentsReply for a connection that wasn't properly initialized")
@@ -130,11 +144,15 @@ class PeerManager(BaseManager):
 
     @on_message(PeerUserInfoRequest.Request)
     async def _on_peer_user_info_request(self, message: PeerUserInfoRequest.Request, connection: PeerConnection):
+
         if not connection.username:
             logger.warning(
                 "got PeerSharesRequest for a connection that wasn't properly initialized")
             return
 
+        if self._settings.users.is_blocked(connection.username, BlockingFlag.INFO):
+            return
+
         description = self._settings.credentials.info.description or ""
         picture = self._settings.credentials.info.picture
 
