@@ -2,9 +2,8 @@
 
 Field metadata:
 
-During (de)serialization the `metadata` parameter of the `dataclasses.field`
-function to control how to perform (de)serialization. See
-`aioslsk.protocol.primitives` for how (de)serialization is performed
+During (de)serialization the ``metadata`` parameter of the ``dataclasses.field``
+function to control how to perform (de)serialization.
 
 These metadata keys are implemented:
 
@@ -32,6 +31,7 @@ These metadata keys are implemented:
       has been fully parsed. If not it will parse this field
 """
 from dataclasses import dataclass, field, Field, fields, is_dataclass
+import enum
 import hashlib
 import logging
 import socket
@@ -40,12 +40,8 @@ from typing_extensions import Self
 from typing import (
     Any,
     ClassVar,
-    Dict,
-    List,
     Optional,
     Protocol,
-    Tuple,
-    Type,
     TypeVar,
     Union
 )
@@ -62,13 +58,21 @@ class Serializable(Protocol):
         ...
 
     @classmethod
-    def deserialize(cls, pos: int, data: bytes) -> Tuple[int, Self]:
+    def deserialize(cls, pos: int, data: bytes) -> tuple[int, Self]:
         ...
 
     def serialize(self, *args, **kwargs) -> bytes:
         ...
 
 
+class AttributeKey(enum.Enum):
+    BITRATE = 0
+    DURATION = 1
+    VBR = 2
+    SAMPLE_RATE = 4
+    BIT_DEPTH = 5
+
+
 def decode_string(value: bytes) -> str:
     try:
         return value.decode('utf-8')
@@ -83,7 +87,7 @@ class uint8(int):
         return self.STRUCT.pack(self)
 
     @classmethod
-    def deserialize(cls, pos: int, data: bytes) -> Tuple[int, int]:
+    def deserialize(cls, pos: int, data: bytes) -> tuple[int, int]:
         return pos + cls.STRUCT.size, cls.STRUCT.unpack_from(data, offset=pos)[0]
 
 
@@ -94,7 +98,7 @@ class uint16(int):
         return self.STRUCT.pack(self)
 
     @classmethod
-    def deserialize(cls, pos: int, data: bytes) -> Tuple[int, int]:
+    def deserialize(cls, pos: int, data: bytes) -> tuple[int, int]:
         return pos + cls.STRUCT.size, cls.STRUCT.unpack_from(data, offset=pos)[0]
 
 
@@ -105,19 +109,20 @@ class uint32(int):
         return self.STRUCT.pack(self)
 
     @classmethod
-    def deserialize(cls, pos: int, data: bytes) -> Tuple[int, int]:
+    def deserialize(cls, pos: int, data: bytes) -> tuple[int, int]:
         return pos + cls.STRUCT.size, cls.STRUCT.unpack_from(data, offset=pos)[0]
 
 
 class uint64(int):
     STRUCT = struct.Struct('<Q')
+    STRUCT_SIZE = STRUCT.size
 
     def serialize(self) -> bytes:
         return self.STRUCT.pack(self)
 
     @classmethod
-    def deserialize(cls, pos: int, data: bytes) -> Tuple[int, int]:
-        return pos + cls.STRUCT.size, cls.STRUCT.unpack_from(data, offset=pos)[0]
+    def deserialize(cls, pos: int, data: bytes) -> tuple[int, int]:
+        return pos + cls.STRUCT_SIZE, cls.STRUCT.unpack_from(data, offset=pos)[0]
 
 
 class int32(int):
@@ -127,7 +132,7 @@ class int32(int):
         return self.STRUCT.pack(self)
 
     @classmethod
-    def deserialize(cls, pos: int, data: bytes) -> Tuple[int, int]:
+    def deserialize(cls, pos: int, data: bytes) -> tuple[int, int]:
         return pos + cls.STRUCT.size, cls.STRUCT.unpack_from(data, offset=pos)[0]
 
 
@@ -139,13 +144,14 @@ class string(str):
         return uint32(len(byte_string)).serialize() + byte_string
 
     @classmethod
-    def deserialize(cls, pos: int, data: bytes) -> Tuple[int, str]:
+    def deserialize(cls, pos: int, data: bytes) -> tuple[int, str]:
         pos_after_len, length = uint32.deserialize(pos, data)
-        value = data[pos_after_len:pos_after_len + length]
+        end_pos = pos_after_len + length
+        value = data[pos_after_len:end_pos]
         if len(value) != length:
             raise Exception(
                 f"expected string with length ({length}), got {len(value)}")
-        return pos_after_len + length, decode_string(value)
+        return end_pos, decode_string(value)
 
 
 class bytearr(bytes):
@@ -155,7 +161,7 @@ class bytearr(bytes):
         return uint32(length).serialize() + bytes(self)
 
     @classmethod
-    def deserialize(cls, pos: int, data: bytes) -> Tuple[int, bytes]:
+    def deserialize(cls, pos: int, data: bytes) -> tuple[int, bytes]:
         pos_after_len, length = uint32.deserialize(pos, data)
         value = data[pos_after_len:pos_after_len + length]
         return pos_after_len + length, value
@@ -169,7 +175,7 @@ class ipaddr(str):
         return self.STRUCT.pack(bytes(reversed(ip_b)))
 
     @classmethod
-    def deserialize(cls, pos: int, data) -> Tuple[int, str]:
+    def deserialize(cls, pos: int, data: bytes) -> tuple[int, str]:
         value = cls.STRUCT.unpack(data[pos:pos + 4])[0]
         ip_addr = socket.inet_ntoa(bytes(reversed(value)))
         return pos + 4, ip_addr
@@ -182,13 +188,13 @@ class boolean(int):
         return self.STRUCT.pack(self)
 
     @classmethod
-    def deserialize(cls, pos: int, data: bytes) -> Tuple[int, bool]:
+    def deserialize(cls, pos: int, data: bytes) -> tuple[int, bool]:
         return pos + cls.STRUCT.size, cls.STRUCT.unpack_from(data, offset=pos)[0]
 
 
 class array(list):
 
-    def serialize(self, element_type: Type[Serializable]) -> bytes:
+    def serialize(self, element_type: type[Serializable]) -> bytes:
         body = uint32(len(self)).serialize()
         is_protocoldc = is_dataclass(element_type)
         for value in self:
@@ -199,7 +205,7 @@ class array(list):
         return body
 
     @classmethod
-    def deserialize(cls, pos: int, data: bytes, element_type: Type[T]) -> Tuple[int, List[T]]:
+    def deserialize(cls, pos: int, data: bytes, element_type: type[T]) -> tuple[int, list[T]]:
         items = []
         pos, array_len = uint32.deserialize(pos, data)
         for _ in range(array_len):
@@ -209,10 +215,10 @@ class array(list):
 
 
 class ProtocolDataclass:
-    """The `ProtocolDataclass` defines a collection of primitives that can be
-    serialized or deserialized. Classes inheriting from this class should use
-    the `@dataclass(order=True)` decorator. The order needs to be kept as the
-    fields definitions will be evaluated during (de)serialization.
+    """The :class:`.ProtocolDataclass` defines a collection of primitives that
+    can be serialized or deserialized. Classes inheriting from this class should
+    use the ``@dataclass(order=True)`` decorator. The order needs to be kept as
+    the fields definitions will be evaluated during (de)serialization.
 
     Example definition:
 
@@ -224,8 +230,8 @@ class ProtocolDataclass:
             password: str = field(metadata={'type': string})
             has_privileges: bool = field(metadata={'type': boolean})
     """
-    _CACHED_FIELDS: Optional[Tuple[Field, ...]] = None
-    """Cache version of the `dataclasses.fields` return for the current class
+    _CACHED_FIELDS: Optional[tuple[Field, ...]] = None
+    """Cache version of the ``dataclasses.fields`` return for the current class
     """
 
     def serialize(self) -> bytes:
@@ -244,7 +250,7 @@ class ProtocolDataclass:
 
             # Serialize
             try:
-                proto_type: Type[Serializable] = obj_field.metadata['type']
+                proto_type: type[Serializable] = obj_field.metadata['type']
             except KeyError:
                 raise Exception(f"no 'type' for field {obj_field.name!r} defined")
 
@@ -274,7 +280,7 @@ class ProtocolDataclass:
             # cause too many issues
             cls._CACHED_FIELDS = fields(cls)  # type: ignore[arg-type]
 
-        field_map: Dict[str, Any] = {}
+        field_map: dict[str, Any] = {}
         for obj_field in cls._CACHED_FIELDS:
             if not cls._field_needs_deserialization(obj_field, field_map, pos, message):
                 continue
@@ -297,7 +303,7 @@ class ProtocolDataclass:
         return pos, cls(**field_map)
 
     @classmethod
-    def _field_needs_deserialization(cls, field: Field, field_map: Dict[str, Any], pos: int, message: bytes):
+    def _field_needs_deserialization(cls, field: Field, field_map: dict[str, Any], pos: int, message: bytes):
         # For if_true and if_false we need to return only if the condition is
         # is false as we still want to check the 'optional' field
         if 'if_true' in field.metadata:
@@ -334,7 +340,7 @@ class ProtocolDataclass:
 
 class MessageDataclass(ProtocolDataclass):
     """Message data class for which protocol messages should inherit from. This
-    takes all behaviour from the `ProtocolDataclass` class but adds:
+    takes all behaviour from the :class:`.ProtocolDataclass` class but adds:
 
     * Prepending the message with length and MESSAGE_ID
     * Optionally the message data will (de)compressed
@@ -343,33 +349,32 @@ class MessageDataclass(ProtocolDataclass):
 
     def serialize(self, compress: bool = False) -> bytes:
         """Serializes the current `MessageDataClass` object and prepends the
-        message length and `MESSAGE_ID`
+        message length and ``MESSAGE_ID``
 
         In case the message needs to be compressed just override this method
-        in the subclass and simply call the super method with `compress=True`
+        in the subclass and simply call the super method with ``compress=True``
 
         :param compress: use gzip compression on the message contents
         """
         message = super().serialize()
 
         if compress:
-            message_len_before = len(message)
             message = zlib.compress(message)
-            logger.debug(f"compressed {message_len_before} to {len(message)} bytes")
 
         message = self.MESSAGE_ID.serialize() + message
         return uint32(len(message)).serialize() + message
 
     @classmethod
     def deserialize(cls, pos: int, message: bytes, decompress: bool = False) -> Self:
-        """Deserializes the passed `message` into an object of the current type
+        """Deserializes the passed ``message`` into an object of the current type
 
         In case the message needs to be decompressed just override this method
-        in the subclass and simply call the super method with `decompress=True`
+        in the subclass and simply call the super method with
+        ``decompress=True``
 
         :param decompress: use gzip decompression on the message contents
         :raise ValueError: if the message_id found the data does not match the
-            `MESSAGE_ID` defined in the current class
+            ``MESSAGE_ID`` defined in the current class
         :return: an object of the current class
         """
         # Parse length and header
@@ -379,16 +384,16 @@ class MessageDataclass(ProtocolDataclass):
             raise ValueError(f"message id mismatch {message_id} != {cls.MESSAGE_ID}")
 
         if decompress:
-            message_len_before = len(message)
             message = zlib.decompress(message[pos:])
-            logger.debug(f"decompressed {message_len_before} to {len(message)} bytes")
-
             pos, obj = super().deserialize(0, message)
         else:
             pos, obj = super().deserialize(pos, message)
 
         if has_unparsed_bytes(pos, message):
-            logger.warning(f"message has {len(message[pos:])} unparsed bytes : {message!r}")
+            logger.warning(
+                "message has %d unparsed bytes : %r",
+                len(message[pos:]), message
+            )
 
         return obj
 
@@ -451,7 +456,7 @@ class FileData(ProtocolDataclass):
     filename: str = field(metadata={'type': string})
     filesize: int = field(metadata={'type': uint64})
     extension: str = field(metadata={'type': string})
-    attributes: List[Attribute] = field(metadata={'type': array, 'subtype': Attribute})
+    attributes: list[Attribute] = field(metadata={'type': array, 'subtype': Attribute})
 
     @classmethod
     def deserialize(cls, pos: int, message: bytes):
@@ -477,11 +482,24 @@ class FileData(ProtocolDataclass):
             array(self.attributes).serialize(Attribute)
         )
 
+    def get_attribute_map(self) -> dict[AttributeKey, int]:
+        """Converts the attribute list to a dictionary. The resulting dictionary
+        will only contain known attributes and attributes present in the list
+        """
+        attribute_dict = {}
+        for attr in self.attributes:
+            try:
+                attribute_dict[AttributeKey(attr.key)] = attr.value
+            except ValueError:
+                pass
+
+        return attribute_dict
+
 
 @dataclass(frozen=True, order=True)
 class DirectoryData(ProtocolDataclass):
     name: str = field(metadata={'type': string})
-    files: List[FileData] = field(metadata={'type': array, 'subtype': FileData})
+    files: list[FileData] = field(metadata={'type': array, 'subtype': FileData})
 
     @classmethod
     def deserialize(cls, pos: int, message: bytes):
