@@ -2,16 +2,19 @@ import asyncio
 from collections import deque
 from functools import partial
 import logging
-from typing import Deque, Dict, List, Optional, Union
+from typing import Optional, Union
 
 from ..base_manager import BaseManager
+from ..constants import DEFAULT_WISHLIST_INTERVAL
 from ..events import (
     on_message,
     build_message_map,
-    EventBus,
     ConnectionStateChangedEvent,
+    EventBus,
     MessageReceivedEvent,
     SearchRequestReceivedEvent,
+    SearchRequestRemovedEvent,
+    SearchRequestSentEvent,
     SearchResultEvent,
     SessionDestroyedEvent,
     SessionInitializedEvent,
@@ -41,7 +44,9 @@ from ..settings import Settings
 from ..shares.manager import SharesManager
 from ..shares.utils import convert_items_to_file_data
 from ..session import Session
+from ..tasks import BackgroundTask, Timer
 from ..transfer.interface import UploadInfoProvider
+from ..user.model import BlockingFlag
 from ..utils import task_counter, ticket_generator
 from .model import ReceivedSearch, SearchResult, SearchRequest, SearchType
 
@@ -56,6 +61,7 @@ class SearchManager(BaseManager):
             self, settings: Settings, event_bus: EventBus,
             shares_manager: SharesManager, upload_info_provider: UploadInfoProvider,
             network: Network):
+
         self._settings: Settings = settings
         self._event_bus: EventBus = event_bus
         self._network: Network = network
@@ -65,20 +71,25 @@ class SearchManager(BaseManager):
         self._ticket_generator = ticket_generator()
         self._session: Optional[Session] = None
 
-        self.received_searches: Deque[ReceivedSearch] = deque(list(), 500)
-        self.requests: Dict[int, SearchRequest] = {}
+        self.received_searches: deque[ReceivedSearch] = deque(
+            list(), maxlen=self._settings.searches.receive.store_amount)
+        self.requests: dict[int, SearchRequest] = {}
 
         # Server variables
         self.search_inactivity_timeout: Optional[int] = None
         self.wishlist_interval: Optional[int] = None
-        self.excluded_search_phrases: List[str] = []
+        self.excluded_search_phrases: list[str] = []
 
         self.register_listeners()
 
         self._MESSAGE_MAP = build_message_map(self)
 
-        self._search_reply_tasks: List[asyncio.Task] = []
-        self._wishlist_task: Optional[asyncio.Task] = None
+        self._search_reply_tasks: list[asyncio.Task] = []
+        self._wishlist_task: BackgroundTask = BackgroundTask(
+            interval=DEFAULT_WISHLIST_INTERVAL,
+            task_coro=self._wishlist_job,
+            name='wishlist-task'
+        )
 
     def register_listeners(self):
         self._event_bus.register(
@@ -94,7 +105,8 @@ class SearchManager(BaseManager):
         """Removes the search request from the client. Incoming results after
         the request has been removed will be ignored
 
-        :param request: `SearchRequest` object or ticket number to remove
+        :param request: :class:`.SearchRequest` object or ticket number to
+            remove
         """
         ticket = request if isinstance(request, int) else request.ticket
         self.requests.pop(ticket)
@@ -102,7 +114,7 @@ class SearchManager(BaseManager):
     async def search(self, query: str) -> SearchRequest:
         """Performs a global search. The results generated by this query will
         stored in the returned object or can be listened to through the
-        `SearchResultEvent` event
+        :class:`.SearchResultEvent` event
 
         :param query: The search query
         :return: An object containing the search request details and results
@@ -112,17 +124,19 @@ class SearchManager(BaseManager):
         await self._network.send_server_messages(
             FileSearch.Request(ticket, query)
         )
-        self.requests[ticket] = SearchRequest(
+        request = SearchRequest(
             ticket=ticket,
             query=query,
             search_type=SearchType.NETWORK
         )
-        return self.requests[ticket]
+        await self._attach_request_timer_and_emit(request)
+
+        return request
 
     async def search_room(self, room: Union[str, Room], query: str) -> SearchRequest:
-        """Performs a search request on the specific user. The results generated
+        """Performs a search request on the specific room. The results generated
         by this query will stored in the returned object or can be listened to
-        through the `SearchResultEvent` event
+        through the :class:`.SearchResultEvent` event
 
         :param room: Room object or name to query
         :param query: The search query
@@ -134,18 +148,20 @@ class SearchManager(BaseManager):
         await self._network.send_server_messages(
             RoomSearch.Request(room_name, ticket, query)
         )
-        self.requests[ticket] = SearchRequest(
+        request = SearchRequest(
             ticket=ticket,
             query=query,
             search_type=SearchType.ROOM,
             room=room_name
         )
-        return self.requests[ticket]
+        await self._attach_request_timer_and_emit(request)
+
+        return request
 
     async def search_user(self, username: str, query: str) -> SearchRequest:
         """Performs a search request on the specific user. The results generated
         by this query will stored in the returned object or can be listened to
-        through the `SearchResultEvent` event
+        through the :class:`.SearchResultEvent` event
 
         :param user: User object or name to query
         :param query: The search query
@@ -156,13 +172,15 @@ class SearchManager(BaseManager):
         await self._network.send_server_messages(
             UserSearch.Request(username, ticket, query)
         )
-        self.requests[ticket] = SearchRequest(
+        request = SearchRequest(
             ticket=ticket,
             query=query,
             search_type=SearchType.USER,
             username=username
         )
-        return self.requests[ticket]
+        await self._attach_request_timer_and_emit(request)
+
+        return request
 
     async def _query_shares_and_reply(self, ticket: int, username: str, query: str):
         """Performs a query on the shares manager and reports the results to the
@@ -172,6 +190,9 @@ class SearchManager(BaseManager):
             logger.warning("not returning search results : no valid session was set")
             return
 
+        if self._settings.users.is_blocked(username, BlockingFlag.SEARCHES):
+            return
+
         visible, locked = self._shares_manager.query(
             query,
             username=username,
@@ -197,7 +218,10 @@ class SearchManager(BaseManager):
         if len(visible) + len(locked) == 0:
             return
 
-        logger.info(f"found {len(visible)}/{len(locked)} results for query {query!r} (username={username!r})")
+        logger.info(
+            "found %d/%d results for query %r (username=%r)",
+            len(visible), len(locked), query, username
+        )
 
         task = asyncio.create_task(
             self._network.send_peer_messages(
@@ -227,44 +251,82 @@ class SearchManager(BaseManager):
 
         except asyncio.CancelledError:
             logger.debug(
-                f"cancelled delivery of search results (ticket={ticket}, username={username}, query={query})")
+                "cancelled delivery of search results (ticket=%d, username=%s, query=%s)",
+                ticket, username, query
+            )
         except Exception as exc:
             logger.warning(
-                f"failed to deliver search results : {exc!r} (ticket={ticket}, username={username}, query={query})")
+                "failed to deliver search results : {exc!r} (ticket=%d, username=%s, query=%s)",
+                exc, ticket, username, query
+            )
         else:
             logger.info(
-                f"delivered search results (ticket={ticket}, username={username}, query={query})")
+                "delivered search results (ticket=%d, username=%s, query=%s)",
+                ticket, username, query
+            )
         finally:
             self._search_reply_tasks.remove(task)
 
-    async def _wishlist_job(self, interval: int):
+    async def _wishlist_job(self):
         """Job handling wishlist queries, this method is intended to be run as
-        a task. This method will run at the given `interval` (returned by the
-        server on start up).
+        a task. This method will run at the given ``interval`` (returned by the
+        server after logon).
         """
-        while True:
-            items = self._settings.searches.wishlist
+        items = self._settings.searches.wishlist
 
-            # Remove all current wishlist searches
-            self.requests = {
-                ticket: qry for ticket, qry in self.requests.items()
-                if qry.search_type != SearchType.WISHLIST
-            }
+        timeout = self._get_wishlist_request_timeout()
 
-            logger.info(f"starting wishlist search of {len(items)} items")
-            # Recreate
-            for item in filter(lambda item: item.enabled, items):
-                ticket = next(self._ticket_generator)
-                self.requests[ticket] = SearchRequest(
-                    ticket,
-                    item.query,
-                    search_type=SearchType.WISHLIST
-                )
-                self._network.queue_server_messages(
-                    WishlistSearch.Request(ticket, item.query)
-                )
+        enabled_items = list(filter(lambda item: item.enabled, items))
+        logger.info("starting wishlist search of %d items", len(enabled_items))
+        # Recreate
+        for item in enabled_items:
+            ticket = next(self._ticket_generator)
 
-            await asyncio.sleep(interval)
+            await self._network.send_server_messages(
+                WishlistSearch.Request(ticket, item.query)
+            )
+
+            request = SearchRequest(
+                ticket,
+                item.query,
+                search_type=SearchType.WISHLIST
+            )
+            request.timer = Timer(
+                timeout=timeout,
+                callback=partial(self._timeout_search_request, request)
+            ) if timeout else None
+            self.requests[ticket] = request
+
+            if request.timer:
+                request.timer.start()
+
+            await self._event_bus.emit(SearchRequestSentEvent(request))
+
+    def _get_wishlist_request_timeout(self) -> int:
+        timeout = self._settings.searches.send.wishlist_request_timeout
+        if self._settings.searches.send.wishlist_request_timeout < 0:
+            if self.wishlist_interval is None:
+                timeout = DEFAULT_WISHLIST_INTERVAL
+            else:
+                timeout = self.wishlist_interval
+
+        return timeout
+
+    async def _attach_request_timer_and_emit(self, request: SearchRequest):
+        self.requests[request.ticket] = request
+
+        if self._settings.searches.send.request_timeout > 0:
+            request.timer = Timer(
+                timeout=self._settings.searches.send.request_timeout,
+                callback=partial(self._timeout_search_request, request)
+            )
+            request.timer.start()
+
+        await self._event_bus.emit(SearchRequestSentEvent(request))
+
+    async def _timeout_search_request(self, request: SearchRequest):
+        del self.requests[request.ticket]
+        await self._event_bus.emit(SearchRequestRemovedEvent(request))
 
     async def _on_message_received(self, event: MessageReceivedEvent):
         message = event.message
@@ -286,7 +348,7 @@ class SearchManager(BaseManager):
             self, message: DistributedServerSearchRequest.Request, connection: PeerConnection):
 
         if message.distributed_code != DistributedSearchRequest.Request.MESSAGE_ID:
-            logger.warning(f"no handling for server search request with code {message.distributed_code}")
+            logger.warning("no handling for server search request with code %d", message.distributed_code)
             return
 
         await self._query_shares_and_reply(message.ticket, message.username, message.query)
@@ -328,10 +390,14 @@ class SearchManager(BaseManager):
         )
         try:
             query = self.requests[message.ticket]
+
         except KeyError:
-            logger.warning(f"search reply ticket does not match any search query : {message.ticket}")
+            logger.warning("search reply ticket does not match any search request : %d", message.ticket)
+
         else:
-            query.results.append(search_result)
+            if self._settings.searches.send.store_results:
+                query.results.append(search_result)
+
             await self._event_bus.emit(SearchResultEvent(query, search_result))
 
         await connection.disconnect(reason=CloseReason.REQUESTED)
@@ -339,12 +405,12 @@ class SearchManager(BaseManager):
     @on_message(WishlistInterval.Response)
     async def _on_wish_list_interval(self, message: WishlistInterval.Response, connection: ServerConnection):
         self.wishlist_interval = message.interval
-        self._cancel_wishlist_task()
 
-        self._wishlist_task = asyncio.create_task(
-            self._wishlist_job(message.interval),
-            name=f'wishlist-job-{task_counter()}'
-        )
+        if task := self._wishlist_task.cancel():
+            await task
+
+        self._wishlist_task.interval = self.wishlist_interval
+        self._wishlist_task.start()
 
     @on_message(ExcludedSearchPhrases.Response)
     async def _on_excluded_search_phrases(self, message: ExcludedSearchPhrases.Response, connection: ServerConnection):
@@ -355,7 +421,7 @@ class SearchManager(BaseManager):
             return
 
         if event.state == ConnectionState.CLOSING:
-            self._cancel_wishlist_task()
+            self._wishlist_task.cancel()
 
     async def _on_session_initialized(self, event: SessionInitializedEvent):
         self._session = event.session
@@ -363,15 +429,7 @@ class SearchManager(BaseManager):
     async def _on_session_destroyed(self, event: SessionDestroyedEvent):
         self._session = None
 
-    def _cancel_wishlist_task(self) -> Optional[asyncio.Task]:
-        task = self._wishlist_task
-        if self._wishlist_task is not None:
-            self._wishlist_task.cancel()
-            self._wishlist_task = None
-            return task
-        return None
-
-    async def stop(self) -> List[asyncio.Task]:
+    async def stop(self) -> list[asyncio.Task]:
         """Cancels all pending tasks
 
         :return: a list of tasks that have been cancelled so that they can be
@@ -383,7 +441,7 @@ class SearchManager(BaseManager):
             task.cancel()
             cancelled_tasks.append(task)
 
-        if (wishlist_task := self._cancel_wishlist_task()) is not None:
+        if wishlist_task := self._wishlist_task.cancel():
             cancelled_tasks.append(wishlist_task)
 
         return cancelled_tasks
