@@ -1,7 +1,8 @@
+from collections.abc import Set
 import os
-from typing import Dict, List, Optional, Set
+from typing import Annotated, Any, Optional, Sequence
 
-from pydantic import BaseModel, Field
+from pydantic import BaseModel, BeforeValidator, Field, model_validator
 from pydantic_settings import BaseSettings
 
 from .constants import (
@@ -11,12 +12,28 @@ from .constants import (
 )
 from .network.network import ListeningConnectionErrorMode
 from .shares.model import DirectoryShareMode
+from .user.model import BlockingFlag
+
+
+def translate_blocked_users(value: Any) -> dict[str, BlockingFlag]:
+    """Converts blocked user set to a dictionary object"""
+    if isinstance(value, dict):
+        return value
+
+    if isinstance(value, (Sequence, Set)):
+        blocked_dict = {}
+        for username in value:
+            blocked_dict[username] = BlockingFlag.ALL
+
+        return blocked_dict
+
+    raise ValueError(f"unable to translate type {type(value)}")
 
 
 class SharedDirectorySettingEntry(BaseModel, validate_assignment=True):
     path: str
     share_mode: DirectoryShareMode = DirectoryShareMode.EVERYONE
-    users: List[str] = Field(default_factory=list)
+    users: list[str] = Field(default_factory=list)
 
 
 class WishlistSettingEntry(BaseModel, validate_assignment=True):
@@ -80,9 +97,41 @@ class CredentialsSettings(BaseModel, validate_assignment=True):
         return bool(self.username) and self.password is not None
 
 
-class SearchSettings(BaseModel, validate_assignment=True):
+class SearchSendSettings(BaseModel, validate_assignment=True):
+    store_results: bool = True
+    request_timeout: int = 0
+    wishlist_request_timeout: int = -1
+
+
+class SearchReceiveSettings(BaseModel, validate_assignment=True):
+    store_amount: int = 500
     max_results: int = 100
-    wishlist: List[WishlistSettingEntry] = Field(default_factory=list)
+
+
+class SearchSettings(BaseModel, validate_assignment=True):
+    send: SearchSendSettings = Field(default_factory=SearchSendSettings)
+    receive: SearchReceiveSettings = Field(default_factory=SearchReceiveSettings)
+    wishlist: list[WishlistSettingEntry] = Field(default_factory=list)
+
+    @model_validator(mode='before')
+    @classmethod
+    def handle_max_result_alias(cls, values: dict):
+        if 'max_results' in values:
+            value = values.pop('max_results')
+            if 'receive' not in values:
+                values['receive'] = {}
+
+            values['receive']['max_results'] = value
+
+        return values
+
+    @property
+    def max_results(self) -> int:
+        return self.receive.max_results
+
+    @max_results.setter
+    def max_results(self, value: int):
+        self.receive.max_results = value
 
 
 class TransferLimitSettings(BaseModel, validate_assignment=True):
@@ -97,28 +146,34 @@ class TransfersSettings(BaseModel, validate_assignment=True):
 class SharesSettings(BaseModel, validate_assignment=True):
     scan_on_start: bool = True
     download: str = os.getcwd()
-    directories: List[SharedDirectorySettingEntry] = Field(default_factory=list)
+    directories: list[SharedDirectorySettingEntry] = Field(default_factory=list)
 
 
 class RoomsSettings(BaseModel, validate_assignment=True):
     auto_join: bool = True
     private_room_invites: bool = True
-    favorites: Set[str] = Field(default_factory=set)
+    favorites: set[str] = Field(default_factory=set)
 
 
 class UsersSettings(BaseModel, validate_assignment=True):
-    friends: Set[str] = Field(default_factory=set)
-    blocked: Set[str] = Field(default_factory=set)
+    friends: set[str] = Field(default_factory=set)
+    blocked: Annotated[
+        dict[str, BlockingFlag],
+        BeforeValidator(translate_blocked_users)
+    ] = Field(default_factory=dict)
+
+    def is_blocked(self, username: str, flag: BlockingFlag) -> bool:
+        return bool(self.blocked.get(username, BlockingFlag.NONE) & flag)
 
 
 class InterestsSettings(BaseModel, validate_assignment=True):
-    liked: Set[str] = Field(default_factory=set)
-    hated: Set[str] = Field(default_factory=set)
+    liked: set[str] = Field(default_factory=set)
+    hated: set[str] = Field(default_factory=set)
 
 
 class DebugSettings(BaseModel, validate_assignment=True):
     search_for_parent: bool = True
-    ip_overrides: Dict[str, str] = Field(default_factory=dict)
+    ip_overrides: dict[str, str] = Field(default_factory=dict)
     log_connection_count: bool = False
 
 
@@ -132,3 +187,33 @@ class Settings(BaseSettings, validate_assignment=True):
     interests: InterestsSettings = Field(default_factory=InterestsSettings)
     transfers: TransfersSettings = Field(default_factory=TransfersSettings)
     debug: DebugSettings = Field(default_factory=DebugSettings)
+
+
+def _main(args):
+    import json
+    import sys
+
+    if args.command == 'generate':
+        settings = Settings(
+            credentials=CredentialsSettings(
+                username=args.username,
+                password=args.password
+            )
+        )
+
+        dump = settings.model_dump(mode='json')
+        json.dump(dump, sys.stdout, indent=2)
+
+
+if __name__ == "__main__":
+    import argparse
+
+    parser = argparse.ArgumentParser(description="Setting file tools")
+    subparsers = parser.add_subparsers(dest='command')
+
+    gen_parser = subparsers.add_parser('generate')
+    gen_parser.add_argument('--username', '-u', default="TestUser")
+    gen_parser.add_argument('--password', '-p', default="Pass0")
+
+    args = parser.parse_args()
+    _main(args)
