@@ -1,5 +1,6 @@
 from aiofiles import os as asyncos
 import asyncio
+from collections.abc import Callable
 from concurrent.futures import Executor
 from functools import partial
 import logging
@@ -9,15 +10,7 @@ import os
 import re
 import sys
 import time
-from typing import (
-    Callable,
-    Dict,
-    List,
-    Optional,
-    Set,
-    Tuple,
-    Union,
-)
+from typing import Optional, Union
 import uuid
 from weakref import WeakSet
 
@@ -28,6 +21,7 @@ from ..events import (
     ScanCompleteEvent,
     SessionInitializedEvent,
     SessionDestroyedEvent,
+    SharedDirectoryChangeEvent,
 )
 from ..exceptions import (
     FileNotFoundError,
@@ -50,7 +44,7 @@ from .utils import convert_items_to_file_data
 
 logger = logging.getLogger(__name__)
 
-ItemAttributes = List[Tuple[int, int]]
+ItemAttributes = list[tuple[int, int]]
 ExecutorFactory = Callable[[], Executor]
 
 _COMPRESSED_FORMATS = [
@@ -69,7 +63,7 @@ _QUERY_CLEAN_PATTERN = re.compile(r"[\W_]")
 
 def scan_directory(
         shared_directory: SharedDirectory,
-        children: Optional[List[SharedDirectory]] = None) -> Set[SharedItem]:
+        children: Optional[list[SharedDirectory]] = None) -> set[SharedItem]:
     """Scans the directory for items to share
 
     Warning: when using ProcessPoolExecutor on this method the returned items
@@ -78,10 +72,10 @@ def scan_directory(
     finishes (see https://stackoverflow.com/a/72726998/1419478). In this case
     you should manually assign it again
 
-    :param shared_directory: `SharedDirectory` instance
-    :param children: list of `SharedDirectory` instances, the items in this list
-        will not be returned and should be scanned individually
-    :return: set of `SharedItem` objects found during the scan
+    :param shared_directory: :class:`.SharedDirectory` instance
+    :param children: list of :class:`.SharedDirectory` instances, the items in
+        this list will not be returned and should be scanned individually
+    :return: set of :class:`.SharedItem` objects found during the scan
     """
     children = children or []
     shared_items = set()
@@ -102,7 +96,7 @@ def scan_directory(
             try:
                 modified = os.path.getmtime(os.path.join(directory, filename))
             except OSError:
-                logger.debug(f"could not get modified time for file {filepath!r}")
+                logger.debug("could not get modified time for file : %r ", filepath)
             else:
                 shared_items.add(
                     SharedItem(shared_directory, subdir, filename, modified)
@@ -112,7 +106,7 @@ def scan_directory(
 
 
 def extract_attributes(filepath: str) -> ItemAttributes:
-    """Attempts to extract attributes from the file at `filepath`. If there was
+    """Attempts to extract attributes from the file at ``filepath``. If there was
     an error attempting to extract the attributes this method will log a warning
     and return an empty list of attributes
     """
@@ -144,7 +138,7 @@ def extract_attributes(filepath: str) -> ItemAttributes:
 
     except mutagen.MutagenError as exc:
         logger.warning(
-            f"failed retrieve audio file metadata. path={filepath!r}", exc_info=exc)
+            "failed retrieve audio file metadata. path=%r", filepath, exc_info=exc)
 
     return attributes
 
@@ -156,11 +150,12 @@ class SharesManager(BaseManager):
             self, settings: Settings, event_bus: EventBus,
             network: Network, cache: Optional[SharesCache] = None,
             executor_factory: Optional[ExecutorFactory] = None):
+
         self._settings: Settings = settings
         self._event_bus: EventBus = event_bus
         self._network: Network = network
-        self._term_map: Dict[str, WeakSet[SharedItem]] = {}
-        self._shared_directories: List[SharedDirectory] = []
+        self._term_map: dict[str, WeakSet[SharedItem]] = {}
+        self._shared_directories: list[SharedDirectory] = []
         self._session: Optional[Session] = None
 
         self.cache: SharesCache = cache if cache else SharesNullCache()
@@ -187,7 +182,7 @@ class SharesManager(BaseManager):
     def generate_alias(self, path: str, offset: int = 0) -> str:
         """Generates a directory alias for the given path, this method will be
         called recursively increasing the offset in case the alias is already
-        found in the `shared_directories`.
+        found in the ``shared_directories``.
 
         The hardware address is mixed in to avoid getting the same alias for the
         same directory on different machines. Admittedly this is a lousy
@@ -222,7 +217,7 @@ class SharesManager(BaseManager):
         if self.executor_factory:
             self.executor = self.executor_factory()
 
-    async def stop(self) -> List[asyncio.Task]:
+    async def stop(self) -> list[asyncio.Task]:
         if self.executor:
             self.executor.shutdown()
             self.executor = None
@@ -241,7 +236,7 @@ class SharesManager(BaseManager):
         will be updated, non-existing directories added, directories that no
         longer exist will be removed
         """
-        new_shared_directories: List[SharedDirectory] = []
+        new_shared_directories: list[SharedDirectory] = []
 
         # Add or update directories
         for directory_entry in self._settings.shares.directories:
@@ -270,20 +265,21 @@ class SharesManager(BaseManager):
         """Read the directories from the cache"""
         logger.info("reading directories from cache")
         directories = self.cache.read()
-        logger.info(f"read {len(directories)} directories from cache")
+        logger.info("read %d directories from cache", len(directories))
         self._shared_directories = directories
 
     def write_cache(self):
         """Write current shared directories to the cache"""
-        logger.info(f"writing {len(self._shared_directories)} directories to cache")
+        directory_count = len(self._shared_directories)
+        logger.info("writing %d directories to cache", directory_count)
         self.cache.write(self._shared_directories)
-        logger.info(f"successfully wrote {len(self._shared_directories)} directories to cache")
+        logger.info("successfully wrote %d directories to cache", directory_count)
 
     def get_download_directory(self) -> str:
         """Gets the absolute path the to download directory configured from the
         settings
 
-        :return: Absolute path of the value of the `shares.download` setting
+        :return: Absolute path of the value of the ``shares.download`` setting
         """
         download_dir = self._settings.shares.download
         return os.path.abspath(download_dir)
@@ -291,7 +287,7 @@ class SharesManager(BaseManager):
     async def create_directory(self, absolute_path: str):
         """Ensures the passed directory exists"""
         if not await asyncos.path.exists(absolute_path):
-            logger.info(f"creating directory : {absolute_path}")
+            logger.info("creating directory : %s", absolute_path)
             await asyncos.makedirs(absolute_path, exist_ok=True)
 
     def rebuild_term_map(self):
@@ -304,24 +300,20 @@ class SharesManager(BaseManager):
         """Builds a list of valid terms for the given shared directory"""
         for item in shared_directory.items:
             self._add_item_to_term_map(item)
-        logger.debug(f"term map contains {len(self._term_map)} terms")
+        logger.debug("term map contains %d terms", len(self._term_map))
 
-    async def get_shared_item(self, remote_path: str, username: Optional[str] = None) -> SharedItem:
-        """Gets a shared item from the cache based on the given file path. If
-        the file does not exist in the `shared_items` or the file is present
-        in the cache but does not exist on disk a :class:`.FileNotFoundError` is
-        raised
+    def get_shared_item_cache(self, remote_path: str, username: Optional[str] = None) -> SharedItem:
+        """Gets a shared item from the cache if it exists
 
-        If a `username` is passed this will also check if the file is locked
+        If a ``username`` is passed this will also check if the file is locked
         and raise a :class:`.FileNotSharedError` if the file is not accessible
         for that user
 
         :param remote_path: the remote_path
         :param username: optional username to check if the file is shared or not
-        :raise FileNotFoundError: filename was not found in shared_items or was found
-            but did not exist on disk
+        :raise FileNotFoundError: filename was not found in shared_items
         :raise FileNotSharedError: file is found, but locked for the given
-            `username`
+            ``username``
         """
         for shared_directory in self._shared_directories:
             try:
@@ -329,25 +321,63 @@ class SharesManager(BaseManager):
             except FileNotFoundError:
                 pass
             else:
-                if not await asyncos.path.exists(item.get_absolute_path()):
-                    raise FileNotFoundError(
-                        f"file with remote_path {remote_path} found in cache but not on disk"
-                    )
-
                 if username and self.is_item_locked(item, username):
-                    raise FileNotSharedError(f"File is not shared to user {username}")
+                    raise FileNotSharedError(f"file is not shared to user {username}")
                 return item
         else:
             raise FileNotFoundError(f"file name {remote_path} not found in cache")
 
+    async def get_shared_item(self, remote_path: str, username: Optional[str] = None) -> SharedItem:
+        """Same as :meth:`get_shared_item_cache` but also checks if the file
+        exists on disk
+
+        :param remote_path: the remote_path
+        :param username: optional username to check if the file is shared or not
+        :raise FileNotFoundError: filename was not found in shared_items or was
+            found but did not exist on disk
+        :raise FileNotSharedError: file is found, but locked for the given
+            ``username``
+        """
+        item = self.get_shared_item_cache(remote_path, username=username)
+        if not await asyncos.path.exists(item.get_absolute_path()):
+            raise FileNotFoundError(
+                f"file with remote_path {remote_path} found in cache but not on disk"
+            )
+        return item
+
+    def find_shared_item_cache(
+            self, remote_path: str, username: Optional[str] = None) -> Optional[SharedItem]:
+        """Equivelant to :meth:`get_shared_item_cache` but returns ``None`` if the
+        shared item is not found
+        """
+        try:
+            return self.get_shared_item_cache(remote_path, username=username)
+        except (FileNotFoundError, FileNotSharedError):
+            return None
+
+    async def find_shared_item(
+            self, remote_path: str, username: Optional[str] = None) -> Optional[SharedItem]:
+        """Equivelant to :meth:`get_shared_item` but returns ``None`` if the
+        shared item is not found
+        """
+        try:
+            return await self.get_shared_item(remote_path, username=username)
+        except (FileNotFoundError, FileNotSharedError):
+            return None
+
     def add_shared_directory(
             self, shared_directory: str,
             share_mode: DirectoryShareMode = DirectoryShareMode.EVERYONE,
-            users: Optional[List[str]] = None) -> SharedDirectory:
-        """Adds a shared directory. This method will call `generate_alias` and
-        add the directory to the directory map. This method will not scan the
-        directory, for scanning see the :meth:`scan`, :meth:`scan_directory_files`
-        and :meth:`scan_directory_file_attributes` methods.
+            users: Optional[list[str]] = None) -> SharedDirectory:
+        """Adds a shared directory. This method will call :meth:`generate_alias`
+        and add the directory to the directory map.
+
+        This method will not scan the directory, however if the directory is a
+        (sub)child of an already registered shared directory the items from that
+        directory will be removed from the parent directory to the new directory
+
+        For scanning see the :meth:`scan`, :meth:`scan_directory_files` and
+        :meth:`scan_directory_file_attributes` methods.
 
         :param shared_directory: path of the shared directory
         :param share_mode: the share mode for the directory
@@ -382,13 +412,16 @@ class SharesManager(BaseManager):
             parent.items -= children
 
         self._shared_directories.append(directory_object)
+
+        self._event_bus.emit_sync(SharedDirectoryChangeEvent(directory_object))
+
         return directory_object
 
     def update_shared_directory(
             self, directory: Union[str, SharedDirectory],
             share_mode: Optional[DirectoryShareMode] = None,
-            users: Optional[List[str]] = None) -> SharedDirectory:
-        """Updates `share_mode` and `users` values for the given directory
+            users: Optional[list[str]] = None) -> SharedDirectory:
+        """Updates ``share_mode`` and ``users`` values for the given directory
 
         :param directory: if a string is given this method will attempt to find
             the shared directory based on the absolute path
@@ -407,6 +440,8 @@ class SharesManager(BaseManager):
         if users is not None:
             shared_directory.users = users
 
+        self._event_bus.emit_sync(SharedDirectoryChangeEvent(shared_directory))
+
         return shared_directory
 
     def remove_shared_directory(self, directory: Union[str, SharedDirectory]) -> SharedDirectory:
@@ -433,7 +468,7 @@ class SharesManager(BaseManager):
 
         And removes the `Music\\Artist_One\\ shared directory the files of that
         directory get returned back to the parent directory. In this case file
-        `song_two.mp3` will be shared with EVERYONE again
+        ``song_two.mp3`` will be shared with EVERYONE again
 
         :param shared_directory: :class:`.SharedDirectory` instance to remove
         :return: the removed directory
@@ -461,13 +496,16 @@ class SharesManager(BaseManager):
 
         self._cleanup_term_map()
 
+        self._event_bus.emit_sync(SharedDirectoryChangeEvent(shared_directory))
+
         return shared_directory
 
     def get_shared_directory(self, directory: str) -> SharedDirectory:
-        """Calculates the absolute path of given `directory` and looks for the
-        matching :class:`SharedDirectory` instance
+        """Calculates the absolute path of given ``directory`` and looks for the
+        matching :class:`.SharedDirectory` instance
 
-        :raise SharedDirectoryError: if the directory is not found
+        :raise SharedDirectoryError: if there is no corresponding
+            :class:`.SharedDirectory` instance registered in this class
         """
         abs_path = os.path.normpath(os.path.abspath(directory))
         for shared_directory in self.shared_directories:
@@ -489,10 +527,10 @@ class SharesManager(BaseManager):
             return True
 
     async def scan_directory_files(self, shared_directory: SharedDirectory):
-        """Scans the files for the given `shared_directory`
+        """Scans the files for the given ``shared_directory``
 
         :param shared_directory: :class:`.SharedDirectory` instance to scan
-        :raise SharedDirectoryError: raised when the passed `shared_directory`
+        :raise SharedDirectoryError: raised when the passed ``shared_directory``
             was not added to the manager
         """
         loop = asyncio.get_running_loop()
@@ -502,9 +540,9 @@ class SharesManager(BaseManager):
                 "attempted to scan directory which was not added to the manager"
             )
 
-        logger.info(f"scheduling scan for directory : {shared_directory!r})")
+        logger.info("scheduling scan for directory : %r", shared_directory)
         try:
-            shared_items: Set[SharedItem] = await loop.run_in_executor(
+            shared_items: set[SharedItem] = await loop.run_in_executor(
                 self.executor,
                 partial(
                     scan_directory,
@@ -513,9 +551,10 @@ class SharesManager(BaseManager):
                 )
             )
         except Exception:
-            logger.exception(f"exception scanning directory : {shared_directory!r}")
+            logger.exception("exception scanning directory : %r", shared_directory)
+
         else:
-            logger.debug(f"scan found {len(shared_items)} files for directory {shared_directory!r}")
+            logger.debug("scan found %d files for directory : %r", len(shared_items), shared_directory)
 
             # When using a ProcessPoolExecutor the `shared_directory` property
             # might be set to None (since objects are cloned instead of passed
@@ -537,20 +576,19 @@ class SharesManager(BaseManager):
         self._cleanup_term_map()
 
     async def scan_directory_file_attributes(self, shared_directory: SharedDirectory):
-        """Scans the file attributes for files in the given `shared_directory`.
-        only files that do not yet have attributes will be scanned
+        """Scans the file attributes for files in the given ``shared_directory``
+        only files that do not have attributes will be scanned
 
         The results of the scan are handled internally and are automatically to
         the :class:`.SharedItem` object for which the scan was performed
 
         :param shared_directory: :class:`.SharedDirectory` instance for which
             the files need to be scanned
-        :return: List of futures for each file that needs to be scanned
         """
         loop = asyncio.get_running_loop()
 
         # Schedule the items on the executor
-        futures: List[asyncio.Future] = []
+        futures: list[asyncio.Future] = []
         for item in shared_directory.items:
             if item.attributes is None:
                 future = loop.run_in_executor(
@@ -563,27 +601,26 @@ class SharesManager(BaseManager):
                 futures.append(future)
 
         logger.debug(
-            f"scheduled {len(futures)} / {len(shared_directory.items)} items "
-            f"for attribute extracting for directory {shared_directory}")
+            "scheduled %d / %d items for attribute extracting for directory %s",
+            len(futures), len(shared_directory.items), shared_directory
+        )
 
         start = time.perf_counter()
 
         await asyncio.gather(*futures, return_exceptions=True)
 
         logger.debug(
-            f"scanned attributes for {len(futures)} items in {time.perf_counter() - start}s")
+            "scanned attributes for %d items in %f s", len(futures), time.perf_counter() - start)
 
     def _extract_attributes_callback(self, item: SharedItem, future: asyncio.Future):
         try:
-            attributes = future.result()
+            item.attributes = future.result()
         except Exception:
             logger.warning("exception fetching shared item attributes")
-        else:
-            item.attributes = attributes
 
     async def scan(self):
         """Scan the files and their attributes for all directories currently
-        defined in the `shared_directories`
+        defined in the ``shared_directories``
 
         This method will emit a :class:`.ScanCompleteEvent` on the event bus
         and report the shares to the server
@@ -602,7 +639,7 @@ class SharesManager(BaseManager):
         ]
         await asyncio.gather(*attribute_futures)
 
-        logger.info(f"completed scan in {time.perf_counter() - start_time} seconds")
+        logger.info("completed scan in %f seconds", time.perf_counter() - start_time)
         folder_count, file_count = self.get_stats()
         await self._event_bus.emit(
             ScanCompleteEvent(folder_count, file_count)
@@ -616,10 +653,10 @@ class SharesManager(BaseManager):
     def query(
             self, query: Union[str, SearchQuery],
             username: Optional[str] = None,
-            excluded_search_phrases: Optional[List[str]] = None) -> Tuple[List[SharedItem], List[SharedItem]]:
-        """Performs a query on the `shared_directories` returning the matching
-        items. If `username` is passed this method will return a list of
-        visible results and list of locked results. If `None` the second list
+            excluded_search_phrases: Optional[list[str]] = None) -> tuple[list[SharedItem], list[SharedItem]]:
+        """Performs a query on the ``shared_directories`` returning the matching
+        items. If ``username`` is passed this method will return a list of
+        visible results and list of locked results. If ``None`` the second list
         will always be empty.
 
         This method makes a first pass using the built in term map and filters
@@ -694,15 +731,15 @@ class SharesManager(BaseManager):
             for excl_phrase in excl_phrases:
                 if excl_phrase in found_item.get_query_path().lower():
                     logger.debug(
-                        f"removing search result '{found_item.get_absolute_path()}' due to "
-                        f"excluded phrase '{excl_phrase}'"
+                        "removing search result %r due to excluded phrase %r",
+                        found_item.get_absolute_path(), excl_phrase
                     )
                     break
 
             else:
                 to_keep.add(found_item)
 
-            if len(to_keep) >= self._settings.searches.max_results:
+            if len(to_keep) >= self._settings.searches.receive.max_results:
                 break
 
         found_items = to_keep
@@ -720,10 +757,10 @@ class SharesManager(BaseManager):
         else:
             return list(found_items), []
 
-    def get_stats(self) -> Tuple[int, int]:
+    def get_stats(self) -> tuple[int, int]:
         """Gets the total amount of shared directories and files.
 
-        :return: directory and file count as a `tuple`
+        :return: directory and file count as a ``tuple``
         """
         file_count = sum(
             len(directory.items) for directory in self._shared_directories
@@ -734,11 +771,11 @@ class SharesManager(BaseManager):
         )
         return dir_count, file_count
 
-    def calculate_download_path(self, remote_path: str) -> Tuple[str, str]:
+    def calculate_download_path(self, remote_path: str) -> tuple[str, str]:
         """Calculates the local download path for a remote path returned by
         another peer.
 
-        :return: tuple of the directory and file name
+        :return: ``tuple`` of the directory and file name
         """
         download_dir = self.get_download_directory()
 
@@ -748,7 +785,7 @@ class SharesManager(BaseManager):
             download_dir
         )
 
-    def get_shared_directories_for_user(self, username: str) -> Tuple[List[SharedDirectory], List[SharedDirectory]]:
+    def get_shared_directories_for_user(self, username: str) -> tuple[list[SharedDirectory], list[SharedDirectory]]:
         public_dirs = []
         locked_dirs = []
         for shared_dir in self._shared_directories:
@@ -759,16 +796,17 @@ class SharesManager(BaseManager):
 
         return public_dirs, locked_dirs
 
-    def create_shares_reply(self, username: str) -> Tuple[List[DirectoryData], List[DirectoryData]]:
+    def create_shares_reply(self, username: str) -> tuple[list[DirectoryData], list[DirectoryData]]:
         """Creates a complete list of the currently shared items as a reply to
         a :class:`.PeerSharesRequest` messages
 
         :param username: username of the user requesting the shares reply, this
             is used to determine the locked results
-        :return: tuple with two lists: public directories and locked directories
+        :return: ``tuple`` with two lists: public directories and locked
+            directories
         """
-        def list_unique_directories(directories: List[SharedDirectory]) -> Dict[Tuple[str, ...], List[SharedItem]]:
-            response_dirs: Dict[Tuple[str, ...], List[SharedItem]] = {}
+        def list_unique_directories(directories: list[SharedDirectory]) -> dict[tuple[str, ...], list[SharedItem]]:
+            response_dirs: dict[tuple[str, ...], list[SharedItem]] = {}
 
             for directory in directories:
                 for item in directory.items:
@@ -784,7 +822,7 @@ class SharesManager(BaseManager):
 
             return response_dirs
 
-        def convert_to_directory_shares(directory_map: Dict[Tuple[str, ...], List[SharedItem]]) -> List[DirectoryData]:
+        def convert_to_directory_shares(directory_map: dict[tuple[str, ...], list[SharedItem]]) -> list[DirectoryData]:
             public_shares = []
             for directory, files in directory_map.items():
                 public_shares.append(
@@ -803,7 +841,7 @@ class SharesManager(BaseManager):
 
         return visible_shares, locked_shares
 
-    def create_directory_reply(self, remote_directory: str) -> List[DirectoryData]:
+    def create_directory_reply(self, remote_directory: str) -> list[DirectoryData]:
         """Lists directory data as a response to a directory request. This will
         not contain any information about subdirectories, only the files within
         that directory
@@ -816,7 +854,7 @@ class SharesManager(BaseManager):
         :return: list of directories. Empty if the directory is not shared, a
             list with one entry if the directory is found
         """
-        items: List[SharedItem] = []
+        items: list[SharedItem] = []
 
         remote_dir_parts = tuple(remote_directory.split('\\'))
         remote_dir_parts_len = len(remote_dir_parts)
@@ -859,7 +897,7 @@ class SharesManager(BaseManager):
             if len(values) > 0
         }
 
-    def _get_parent_directories(self, shared_directory: SharedDirectory) -> List[SharedDirectory]:
+    def _get_parent_directories(self, shared_directory: SharedDirectory) -> list[SharedDirectory]:
         """Returns a list of parent shared directories. The parent directories
         will be sorted by length of the absolute path (longest last)
         """
@@ -878,7 +916,8 @@ class SharesManager(BaseManager):
         return children
 
     def is_directory_locked(self, directory: SharedDirectory, username: str) -> bool:
-        """Checks if the shared directory is locked for the given `username`"""
+        """Checks if the shared directory is locked for the given ``username``
+        """
         if directory.share_mode == DirectoryShareMode.FRIENDS:
             return username not in self._settings.users.friends
         elif directory.share_mode == DirectoryShareMode.USERS:
@@ -886,17 +925,16 @@ class SharesManager(BaseManager):
         return False
 
     def is_item_locked(self, item: SharedItem, username: str) -> bool:
-        """Checks if the shared item is locked for the given `username`"""
+        """Checks if the shared item is locked for the given ``username``"""
         return self.is_directory_locked(item.shared_directory, username)
 
     async def report_shares(self):
         """Reports the shares amount to the server"""
         if not self._session:
-            logger.warning("attempted to report shares without valid session")
             return
 
         folder_count, file_count = self.get_stats()
-        logger.debug(f"reporting shares ({folder_count=}, {file_count=})")
+        logger.debug("reporting shares (folder_count=%d, file_count=%d)", folder_count, file_count)
         await self._network.send_server_messages(
             SharedFoldersFiles.Request(
                 shared_folder_count=folder_count,
