@@ -1,7 +1,7 @@
 from dataclasses import dataclass, field
 import enum
 import os
-from typing import List, Optional, Set, Tuple, Union
+from typing import Optional, Union
 
 from ..exceptions import FileNotFoundError
 from .utils import normalize_remote_path
@@ -18,25 +18,35 @@ class DirectoryShareMode(enum.Enum):
 
 @dataclass(eq=True, unsafe_hash=True)
 class SharedDirectory:
+    """Class representing a directory that is explicitly shared by the user.
+    This class holds information on how the directory is shared and is a
+    container for shared items within this directory.
+    """
     directory: str
     absolute_path: str
     alias: str
     share_mode: DirectoryShareMode = field(default=DirectoryShareMode.EVERYONE, compare=False, hash=False)
-    users: List[str] = field(default_factory=list, compare=False, hash=False)
-    items: Set['SharedItem'] = field(default_factory=set, init=False, compare=False, hash=False, repr=False)
+    users: list[str] = field(default_factory=list, compare=False, hash=False)
+    items: set['SharedItem'] = field(default_factory=set, init=False, compare=False, hash=False, repr=False)
 
     def is_parent_of(self, directory: Union[str, 'SharedDirectory']) -> bool:
-        """Returns true if the current directory is a parent of the passed
+        """Returns true if the current directory is any parent of the passed
         shared directory
 
-        :param directory: directory to check
+        :param directory: directory to test, if this directory is of ``str``
+            type it should be an absolute path. If it is of
+            :class:`.SharedDirectory` type the ``absolute_path`` will be taken
         """
         path = directory if isinstance(directory, str) else directory.absolute_path
         return os.path.commonpath([path, self.absolute_path]) == self.absolute_path
 
     def is_child_of(self, directory: Union[str, 'SharedDirectory']) -> bool:
-        """Returns true if the passed directory is a child of the current
+        """Returns true if the passed directory is a (sub)child of the current
         directory
+
+        :param directory: directory to test, if this directory is of ``str``
+            type it should be an absolute path. If it is of
+            :class:`.SharedDirectory` type the ``absolute_path`` will be taken
         """
         path = directory if isinstance(directory, str) else directory.absolute_path
         return os.path.commonpath([self.absolute_path, path]) == path
@@ -45,8 +55,8 @@ class SharedDirectory:
         return '@@' + self.alias
 
     def get_item_by_remote_path(self, remote_path: str) -> 'SharedItem':
-        """Returns the `SharedItem` instance belonging to the passed
-        `remote_path`
+        """Returns the :class:`.SharedItem` instance belonging to the passed
+        ``remote_path``
 
         :raise FileNotFoundError: when the item cannot be found in the set of
             `items`
@@ -58,13 +68,13 @@ class SharedDirectory:
             raise FileNotFoundError(
                 f"file with remote path {remote_path!r} not found in directory {self!r}")
 
-    def get_items_for_directory(self, directory: 'SharedDirectory') -> Set['SharedItem']:
-        """Gets items that are part of given directory"""
-        items = set()
-        for item in self.items:
-            if os.path.commonpath([directory.absolute_path, item.get_absolute_path()]) == directory.absolute_path:
-                items.add(item)
-        return items
+    def get_items_for_directory(self, directory: 'SharedDirectory') -> set['SharedItem']:
+        """Gets items in the current directory that are part of given directory
+        """
+        return {
+            item for item in self.items
+            if os.path.commonpath([directory.absolute_path, item.get_absolute_path()]) == directory.absolute_path
+        }
 
 
 @dataclass(eq=True, unsafe_hash=True)
@@ -73,7 +83,7 @@ class SharedItem:
     subdir: str
     filename: str
     modified: float
-    attributes: Optional[List[Tuple[int, int]]] = field(
+    attributes: Optional[list[tuple[int, int]]] = field(
         default=None,
         init=False,
         compare=False,
@@ -101,7 +111,7 @@ class SharedItem:
         return normalize_remote_path(
             '@@' + os.path.join(self.shared_directory.alias, self.subdir))
 
-    def get_remote_directory_path_parts(self) -> Tuple[str, ...]:
+    def get_remote_directory_path_parts(self) -> tuple[str, ...]:
         """Returns the remote directory path split into parts"""
         return tuple(self.get_remote_directory_path().split('\\'))
 
