@@ -5,7 +5,7 @@ from dataclasses import dataclass
 from enum import Enum
 import logging
 import time
-from typing import Deque, List, Optional, Tuple
+from typing import Optional
 
 from .state import TransferState, TransferStateListener, VirginState
 
@@ -16,6 +16,23 @@ SPEED_LOG_INTERVAL = 0.1
 SPEED_LOG_ENTRIES = 30
 
 
+class FailReason:
+    """Definition of reasons for which an transfer queue request or transfer
+    request was rejected
+    """
+    CANCELLED = 'Cancelled'
+    COMPLETE = 'Complete'
+    QUEUED = 'Queued'
+    FILE_NOT_SHARED = 'File not shared.'
+    FILE_READ_ERROR = 'File read error.'
+
+
+class AbortReason:
+    REQUESTED = 'Requested'
+    BLOCKED = 'Blocked'
+    FILE_NOT_SHARED = 'File not shared'
+
+
 class TransferDirection(Enum):
     UPLOAD = 0
     DOWNLOAD = 1
@@ -24,7 +41,7 @@ class TransferDirection(Enum):
 @dataclass(frozen=True, eq=True)
 class TransferProgressSnapshot:
     """Represents the current progress of a transfer, used for reporting
-    progress back to the user through a :class:`aioslsk.events.TransferProgressEvent`
+    progress back to the user through a :class:`.TransferProgressEvent`
     """
     state: TransferState.State
     bytes_transfered: int
@@ -35,6 +52,10 @@ class TransferProgressSnapshot:
     """
     start_time: Optional[float] = None
     complete_time: Optional[float] = None
+    fail_reason: Optional[str] = None
+    """Optional transfer fail reason"""
+    abort_reason: Optional[str] = None
+    """Optional transfer abort reason"""
 
 
 class Transfer:
@@ -51,28 +72,27 @@ class Transfer:
     def __init__(self, username: str, remote_path: str, direction: TransferDirection):
         self.state: TransferState = VirginState(self)
 
+        self.direction: TransferDirection = direction
+        """Determines whether this transfer is an upload or a download"""
         self.username: str = username
         """Username of the peer"""
         self.remote_path: str = remote_path
         """Remote path, this is the path that is shared between peers"""
         self.local_path: Optional[str] = None
         """Absolute path to the file on disk"""
-        self.direction: TransferDirection = direction
-        """Determines whether this transfer is an upload or a download"""
 
         self.remotely_queued: bool = False
-        """Indicates whether the transfer queue message was received by the peer
+        """Indicates whether the transfer queue message was received by the peer.
+        This only used for downloads
         """
         self.place_in_queue: Optional[int] = None
         self.fail_reason: Optional[str] = None
         """Reason for failure in case the transfer has failed"""
+        self.abort_reason: Optional[str] = None
+        """Reason for why transfer was aborted"""
 
         self.filesize: Optional[int] = None
         """Filesize in bytes"""
-        self._offset: int = 0
-        """Offset used for resuming downloads. This offset will be used and
-        reset by the :meth:`read` method of this object
-        """
 
         self.bytes_transfered: int = 0
         """Amount of bytes transfered"""
@@ -96,17 +116,30 @@ class Transfer:
         """Snapshot of the transfer progress. Used internally to report progress
         at set intervals
         """
-        self._speed_log: Deque[Tuple[float, int]] = deque(maxlen=SPEED_LOG_ENTRIES)
+        self._speed_log: deque[tuple[float, int]] = deque(maxlen=SPEED_LOG_ENTRIES)
         self._remotely_queue_task: Optional[asyncio.Task] = None
         self._transfer_task: Optional[asyncio.Task] = None
         self._state_lock: asyncio.Lock = asyncio.Lock()
-        self.state_listeners: List[TransferStateListener] = []
+        self.state_listeners: list[TransferStateListener] = []
 
-    def __setstate__(self, obj_state):
+    def __setstate__(self, obj_state: dict):
         """Called when unpickling"""
+        # Remove variables that are no longer used
+        obj_state.pop('_offset', None)
+
+        # Add variables that might not be in cache
+        obj_state['state'] = TransferState.init_from_state(obj_state['state'], self)
+
+        if 'abort_reason' not in obj_state:
+            obj_state['abort_reason'] = None
+
+        if obj_state['abort_reason'] is None:
+            if obj_state['state'].VALUE == TransferState.ABORTED:
+                obj_state['abort_reason'] = AbortReason.REQUESTED
+
         self.__dict__.update(obj_state)
 
-        self.__dict__['state'] = TransferState.init_from_state(obj_state['state'], self)
+        # Variables that are not stored in cache
         self._speed_log = deque(maxlen=SPEED_LOG_ENTRIES)
         self._remotely_queue_task = None
         self._transfer_task = None
@@ -127,17 +160,24 @@ class Transfer:
 
         return obj_state
 
-    def reset_progress(self):
-        """Resets progress of the entire transfer"""
-        self.reset_times()
+    def reset_local_vars(self):
+        """Resets the local file variables"""
+        self.local_path = None
+        self.filesize = None
+
+    def reset_progress_vars(self):
         self.bytes_transfered = 0
         self._offset = 0
-        self.local_path = None
-        self.fail_reason = None
-        self.remotely_queued = False
-        self.place_in_queue = None
 
-    def reset_times(self):
+    def reset_queue_vars(self):
+        """Reset all variables when the """
+        self.place_in_queue = None
+        self.remotely_queued = False
+
+        self.reset_queue_attempts()
+        self.reset_upload_request_attempts()
+
+    def reset_time_vars(self):
         """Clear all time related variables"""
         self.start_time = None
         self.complete_time = None
@@ -145,15 +185,15 @@ class Transfer:
 
     def set_start_time(self):
         """Set the start time, clear the complete time"""
-        self._speed_log = deque(maxlen=SPEED_LOG_ENTRIES)
         self.start_time = time.time()
         self.complete_time = None
+        self._speed_log = deque(maxlen=SPEED_LOG_ENTRIES)
 
     def set_complete_time(self):
         """Set the complete time only if the start time has not been set"""
         if self.start_time is not None:
-            self._speed_log = deque(maxlen=SPEED_LOG_ENTRIES)
             self.complete_time = time.time()
+            self._speed_log = deque(maxlen=SPEED_LOG_ENTRIES)
 
     def increase_queue_attempts(self):
         self.queue_attempts += 1
@@ -167,17 +207,20 @@ class Transfer:
         self.upload_request_attempts += 1
         self.last_upload_request_attempt = time.monotonic()
 
-    def reset_upload_request_attempt(self):
+    def reset_upload_request_attempts(self):
         self.upload_request_attempts = 0
         self.last_upload_request_attempt = 0.0
 
     async def transition(self, state: TransferState):
         """Transitions the state of the transfer and notifies the
-        `state_listeners`. This is an internal method and should not be used
+        ``state_listeners``. This is an internal method and should not be used
         directly.
         """
         old_state = self.state
-        logger.debug(f"transitioning transfer state from {old_state.VALUE.name} to {state.VALUE.name}")
+        logger.debug(
+            "transitioning transfer state from %s to %s",
+            old_state.VALUE.name, state.VALUE.name
+        )
         self.state = state
 
         for listener in self.state_listeners:
@@ -185,13 +228,6 @@ class Transfer:
                 self, old_state.VALUE, self.state.VALUE
             )
 
-    def set_offset(self, offset: int):
-        self._offset = offset
-        self.bytes_transfered = offset
-
-    def get_offset(self) -> int:
-        return self._offset
-
     def get_speed(self) -> float:
         """Retrieve the speed of the transfer
 
@@ -213,7 +249,7 @@ class Transfer:
             oldest_time = self._speed_log[0][0]
 
             transfer_duration = current_time - oldest_time
-            if current_time - oldest_time == 0.0:
+            if transfer_duration == 0.0:
                 return 0.0
 
             return bytes_transfered / transfer_duration
@@ -276,7 +312,7 @@ class Transfer:
     def is_transfered(self) -> bool:
         return self.filesize == self.bytes_transfered
 
-    def get_tasks(self) -> List[asyncio.Task]:
+    def get_tasks(self) -> list[asyncio.Task]:
         tasks = []
         if self._remotely_queue_task is not None:
             tasks.append(self._remotely_queue_task)
@@ -284,7 +320,7 @@ class Transfer:
             tasks.append(self._transfer_task)
         return tasks
 
-    def cancel_tasks(self) -> List[asyncio.Task]:
+    def cancel_tasks(self) -> list[asyncio.Task]:
         """Cancels all tasks for the transfer, this method returns the tasks
         which have been cancelled
         """
@@ -306,7 +342,9 @@ class Transfer:
             bytes_transfered=self.bytes_transfered,
             speed=self.get_speed(),
             start_time=self.start_time,
-            complete_time=self.complete_time
+            complete_time=self.complete_time,
+            fail_reason=self.fail_reason,
+            abort_reason=self.abort_reason
         )
         self.progress_snapshot = snapshot
         return snapshot
@@ -324,6 +362,7 @@ class Transfer:
     def __eq__(self, other: object):
         if not isinstance(other, Transfer):
             return NotImplemented
+
         other_vars = (other.remote_path, other.username, other.direction, )
         own_vars = (self.remote_path, self.username, self.direction, )
         return other_vars == own_vars
