@@ -1,7 +1,7 @@
 from __future__ import annotations
 from dataclasses import dataclass, field
-from enum import auto, Enum, Flag
-from typing import Optional, Set
+from enum import auto, Enum, Flag, IntFlag
+from typing import Optional
 from ..protocol.primitives import UserStats
 
 
@@ -38,6 +38,27 @@ class TrackingFlag(Flag):
     """Tracking because the user is a friend"""
 
 
+class BlockingFlag(IntFlag):
+    NONE = 0
+    PRIVATE_MESSAGES = 1
+    """Blocks private messages sent by the user"""
+    ROOM_MESSAGES = 2
+    """Blocks room messages sent by the user"""
+    SEARCHES = 4
+    """Blocks incoming searches from the user"""
+    SHARES = 8
+    """Blocks shares, directory listing requests"""
+    INFO = 16
+    UPLOADS = 32
+    """Blocks all uploads and upload requests"""
+
+    # Aliases
+    IGNORE = PRIVATE_MESSAGES | ROOM_MESSAGES
+    """Ignore all messaging"""
+    ALL = IGNORE | SEARCHES | SHARES | INFO | UPLOADS
+    """Fully blocks the user: includes messaging, shares requests, uploads"""
+
+
 @dataclass
 class User:
     name: str
@@ -65,8 +86,8 @@ class User:
     upload_permissions: UploadPermissions = UploadPermissions.UNKNOWN
 
     # Interests
-    interests: Set[str] = field(default_factory=set)
-    hated_interests: Set[str] = field(default_factory=set)
+    interests: set[str] = field(default_factory=set)
+    hated_interests: set[str] = field(default_factory=set)
 
     def update_from_user_stats(self, user_stats: UserStats):
         self.avg_speed = user_stats.avg_speed
@@ -115,7 +136,22 @@ class ChatMessage:
     timestamp: int
     user: User
     message: str
-    is_admin: bool
+    is_direct: bool
 
     def is_server_message(self) -> bool:
-        return self.is_admin and self.user.name == 'server'
+        return self.user.name == 'server'
+
+    @property
+    def is_admin(self) -> bool:
+        """Only kept to keep backward compatibility. Use ``is_direct``
+
+        This property does not actually represent whether the message was sent
+        by an admin. Instead it represents whether the message was sent directly
+        or was queued on the server before being sent (example, when user was
+        offline and came back online)
+        """
+        return self.is_direct
+
+    @is_admin.setter
+    def is_admin(self, value: bool):
+        self.is_direct = value
