@@ -1,7 +1,7 @@
+from collections.abc import Generator
 import itertools
 import logging
 import re
-from typing import Generator, List
 
 from .constants import PATH_SEPERATOR_PATTERN
 from .protocol.primitives import Attribute
@@ -20,16 +20,16 @@ def try_decoding(value: bytes):
         try:
             return value.decode('cp1252')
         except Exception:
-            logger.warning(f"failed to decode string {value!r}")
+            logger.warning("failed to decode string : %r", value)
             raise
 
 
-def split_remote_path(path: str) -> List[str]:
+def split_remote_path(path: str) -> list[str]:
     """Splits a remote path into parts. Empty parts will be filtered out"""
     return [part for part in re.split(PATH_SEPERATOR_PATTERN, path) if part]
 
 
-def get_duration(attributes: List[Attribute]) -> str:
+def get_duration(attributes: list[Attribute]) -> str:
     duration = ''
     for attr in attributes:
         if attr.key == 1:
@@ -40,7 +40,8 @@ def get_duration(attributes: List[Attribute]) -> str:
     return duration
 
 
-def get_attribute_string(attributes: List[Attribute]) -> str:
+def get_attribute_string(attributes: list[Attribute]) -> str:
+    """Returns an attributes list to a string (excludes duration)"""
     attr_str = []
     for attr in attributes:
         if attr.key == 0:
@@ -57,7 +58,7 @@ def get_attribute_string(attributes: List[Attribute]) -> str:
 
 def ticket_generator(initial: int = 1) -> Generator[int, None, None]:
     """Generator for tickets to be used in various protocol messages. The
-    generator will be reset to the `initial` value once the value would exceed
+    generator will be reset to the ``initial`` value once the value would exceed
     2 ^ 32
     """
     idx = initial
